cbuffer cbChangesEveryFrame
{
	float g_scale;				// scale factor 
	float2 g_pan;				// panning 
	float g_diffX;				// x size of diffusion texture
	float g_diffY;				// y size of diffusion texture
	float g_polySize;			// size of the curve polygons (2 is screen width/height)
	float g_showLayers;			// show the layers or not
	float g_blurScale;			// 
	float g_colLScale;
	float g_colRScale;
	float cmpColor;
};

Texture2D g_inTex;	// input texture (containing lines with north east south west colors)
Texture2D g_inTex2;	// this will be used for constraint blurring
Texture2D g_distDirTex;	// diffusion texture 
Texture2D g_paletteTex;	// texture showing the palette
Texture2D g_mousePointerTex;	// texture showing the palette



#define M_PI 3.1415926
#define zMax 1.0

SamplerState PointSampler
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = CLAMP;
    AddressV = CLAMP;
    AddressW = CLAMP;
};


SamplerState LinearSampler
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = CLAMP;
    AddressV = CLAMP;
    AddressW = CLAMP;
};



struct VS_SIMPLE_INPUT
{
    float2 pos		: POSITION;
};


struct VS_INPUT
{
    float3 Pos		: POSITION;
    float2 Tex		: TEXTURE0;
};

struct PS_INPUT
{
	float4 oPosition : SV_POSITION;
    float2 Tex		 : TEXTURE1;
};

struct PS_OUTPUT
{
	float4 oColor	: SV_Target0;
//	float  oDepth	: SV_Depth;
};


struct PS_OUTPUT_DEPTH
{
	float4 oColor	: SV_Target0;
	float  oDepth	: SV_Depth;
};


struct VS_CURVE_INPUT
{
    float2 pos		: POSITION;
    float4 col0		: TEXTURE0;
    float4 col1		: TEXTURE1;
    float2 nb		: TEXTURE2;
//	float4 IDw		: TEXTURE3;  // ID and weight of this vertex
};

struct VS_CURVE_CPU_INPUT
{
    float2 pos		: POSITION;
    float4 col00	: TEXTURE0;
    float4 col01	: TEXTURE1;
    float4 col10	: TEXTURE2;
    float4 col11	: TEXTURE3;
	float2 nb		: TEXTURE4;
	float4 IDw		: TEXTURE5;  // colors ID and weight for this vertex
	float4 IDwB		: TEXTURE6;  // blur ID and weight for this vertex
	float2 IDwPos	: TEXTURE7;  // Bspline ID and weight for this vertex
};



struct PS_CURVE_INPUT
{
	float4 oPosition : SV_POSITION;
    float4 col		 : TEXTURE0;	//contains blurr in w component
    float4 distdir	 : TEXTURE1;    //distance(x) and direction(yz) to the closest border, will be used for diffusion and anti-aliasing
    float4 oCol		 : TEXTURE2;	//color on the other side of of the curve (used for antialiasing)
	float2 pos		 : TEXTURE3;	// current position, will be used for distance computation
};


struct VS_CONTROLPOINTINPUT
{
    float2 pos	: POSITION;
    float3 col	: TEXTURE0;
};

struct PS_CONTROLPOINTINPUT
{
	float4 oPosition : SV_POSITION;
	float3 col		 : TEXTURE0;
	float2 tex		 : TEXTURE1;
};


// ================================================================================
//
// Vertex Shader
//
// ================================================================================


VS_CURVE_INPUT CurveVS(VS_CURVE_CPU_INPUT In)
{
	VS_CURVE_INPUT Out;

    Out.pos = In.pos*g_scale+g_pan;
	if (In.nb.x < 9999.0)
	    Out.nb = In.nb*g_scale+g_pan;
	else
		Out.nb = float2(10000.0,10000.0);
	
	Out.col0.xyz = In.col00.xyz+(In.col00.xyz-0.5)*g_colLScale;
	Out.col1.xyz = In.col01.xyz+(In.col01.xyz-0.5)*g_colRScale;
	if (g_showLayers > 0.5)
	{
		Out.col0.xyz = In.col10.xyz+(In.col10.xyz-0.5)*g_colLScale;
		Out.col1.xyz = In.col11.xyz+(In.col11.xyz-0.5)*g_colRScale;
	}
	Out.col0.w = In.col00.w*g_blurScale;//+0.01; //for anti aliasing but this slows down
	Out.col1.w = In.col01.w*g_blurScale;//+0.01; //for anti aliasing but this slows down

	// for influence display and image fitting
	if (cmpColor>=0)
	{
		Out.col0.xyz = 0;
		Out.col1.xyz = 0;
		float3 wCol = (1-g_showLayers)*float3(0,0,1)+g_showLayers*float3(1,0,0);  // boundary color is in red channel, center color is in blue channel
		if (cmpColor == In.IDw.x)
			Out.col0.xyz = In.IDw.y*wCol;
		else if (cmpColor == In.IDw.z)
			Out.col1.xyz = In.IDw.w*wCol;
		else if (cmpColor == In.IDw.x+1)
			Out.col0.xyz = (1-In.IDw.y)*wCol;
		else if (cmpColor == In.IDw.z+1)
			Out.col1.xyz = (1-In.IDw.w)*wCol;
		Out.col0.y = 0.25;
		Out.col1.y = 0.25;
	}
    return Out;
}



VS_SIMPLE_INPUT BlurDistVS(VS_SIMPLE_INPUT In)
{
	return In;
}


PS_INPUT ScreenQuad(VS_INPUT In)
{
	PS_INPUT Out;
    Out.oPosition = float4(In.Pos.x, -In.Pos.y,+0.5,1.0);
    Out.Tex = In.Tex;
    return Out;
}


PS_INPUT CurveOverlayVS(VS_CURVE_CPU_INPUT In)
{
	PS_INPUT Out;
	Out.oPosition = float4(In.pos*g_scale+g_pan,0.5,1.0);
	Out.Tex = float2(0,0);
    return Out;
}


VS_CONTROLPOINTINPUT ControlPointVS(VS_CONTROLPOINTINPUT In)
{
    return In;
}




// ================================================================================
//
// Geometry Shader
//
// ================================================================================

[maxvertexcount(4)]
void ControlPointGS( point VS_CONTROLPOINTINPUT input[1], inout TriangleStream<PS_CONTROLPOINTINPUT> tStream )
{
	PS_CONTROLPOINTINPUT p0;
	p0.col = input[0].col;
	p0.oPosition.zw = float2(0.5,1.0);
	float scale = g_scale;
	p0.oPosition.xy = input[0].pos+float2(-scale/g_diffX,-scale/g_diffY);
	p0.tex = float2(-1,-1);
	tStream.Append( p0 );
	p0.oPosition.xy = input[0].pos+float2(+scale/g_diffX,-scale/g_diffY);
	p0.tex = float2(+1,-1);
	tStream.Append( p0 );
	p0.oPosition.xy = input[0].pos+float2(-scale/g_diffX,+scale/g_diffY);
	p0.tex = float2(-1,+1);
	tStream.Append( p0 );
	p0.oPosition.xy = input[0].pos+float2(+scale/g_diffX,+scale/g_diffY);
	p0.tex = float2(+1,+1);
	tStream.Append( p0 );
	tStream.RestartStrip();
}



[maxvertexcount(4)]
void BlurDistGSCenter( point VS_SIMPLE_INPUT input[1], inout TriangleStream<PS_INPUT> tStream )
{
	PS_INPUT p0;
	float bMask = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y), 0).w;
	if (bMask<=0.01) // if we are at a left border
	{
		float2 s = input[0].pos.xy+float2(-0.5/g_diffX,-0.5/g_diffY);
		float2 dX = float2(1.0/g_diffX,0);
		float2 dY = float2(0,1.0/g_diffY);
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX+dY, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX+dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	return;
}



[maxvertexcount(4)]
void BlurDistGSLeftRight( point VS_SIMPLE_INPUT input[1], inout TriangleStream<PS_INPUT> tStream )
{
	PS_INPUT p0;
	float bMask = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y), 0).w;
	float bMaskR = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y)+float2(1.0/g_diffX,0), 0).w;
	if ((bMask>0.01) && (bMaskR<=0.01)) // if we are at a left border
	{
		float2 s = input[0].pos.xy+float2(0.5/g_diffX,-0.5/g_diffY);
		float2 dX = float2(-1,0);
		float2 dY = float2(0,1.0/g_diffY);
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX+dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	else if ((bMask<=0.01) && (bMaskR>0.01)) // if we are at a right border
	{
		float2 s = input[0].pos.xy+float2(0.5/g_diffX,-0.5/g_diffY);
		float2 dX = float2( 1,0);
		float2 dY = float2(0,1.0/g_diffY);
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX+dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	return;
}


[maxvertexcount(4)]
void BlurDistGSTopbottom( point VS_SIMPLE_INPUT input[1], inout TriangleStream<PS_INPUT> tStream )
{
	PS_INPUT p0;
	float bMask = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y), 0).w;
	float bMaskT = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y)+float2(0,-1.0/g_diffY), 0).w;
	if ((bMask>0.01) && (bMaskT<=0.01)) // if we are at a left border
	{
		float2 s = input[0].pos.xy+float2(-0.5/g_diffX,+0.5/g_diffY);
		float2 dX = float2(1.0/g_diffX,0);
		float2 dY = float2(0,-1.0);
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX+dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	else if ((bMask<=0.01) && (bMaskT>0.01)) // if we are at a right border
	{
		float2 s = input[0].pos.xy+float2(-0.5/g_diffX,+0.5/g_diffY);
		float2 dX = float2(1.0/g_diffX,0);
		float2 dY = float2(0,1.0);
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dX+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX+dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	return;
}


[maxvertexcount(4)]
void BlurDistGSTL( point VS_SIMPLE_INPUT input[1], inout TriangleStream<PS_INPUT> tStream )
{
	PS_INPUT p0;
	float bMask = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y), 0).w;
	float bMaskTL = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y)+float2(-1.0/g_diffX,1.0/g_diffY), 0).w;
	if ((bMask>0.01) && (bMaskTL<=0.01))
	{
		float2 s = input[0].pos.xy+float2(-0.5/g_diffX,-0.5/g_diffY);
		float2 dX = float2(-1.0,0);
		float2 dY = float2(0,1.0);
		p0.oPosition = float4( s+dX, 1, 1.0);
		p0.Tex = p0.oPosition.xy-s;
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		tStream.Append( p0 );
		p0.oPosition = float4( s, 0, 1.0);
		p0.Tex = p0.oPosition.xy-s;
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY+dX, 1.4142, 1.0);
		p0.Tex = p0.oPosition.xy-s;
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 1, 1.0);
		p0.Tex = p0.oPosition.xy-s;
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	return;
}



[maxvertexcount(4)]
void BlurDistGSBR( point VS_SIMPLE_INPUT input[1], inout TriangleStream<PS_INPUT> tStream )
{
	PS_INPUT p0;
	float bMask = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y), 0).w;
	float bMaskBR = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y)+float2( 1.0/g_diffX, -1.0/g_diffY), 0).w;
	if ((bMask>0.01) && (bMaskBR<=0.01))
	{
		float2 s = input[0].pos.xy+float2( 0.5/g_diffX, 0.5/g_diffY);
		float2 dX = float2( 1.0,0);
		float2 dY = float2(0, -1.0);
		p0.oPosition = float4( s+dX, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY+dX, 1.4142, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY+dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	return;
}


[maxvertexcount(4)]
void BlurDistGSTR( point VS_SIMPLE_INPUT input[1], inout TriangleStream<PS_INPUT> tStream )
{
	PS_INPUT p0;
	float bMask = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y), 0).w;
	float bMaskTL = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y)+float2(1.0/g_diffX,1.0/g_diffY), 0).w;
	if ((bMask>0.01) && (bMaskTL<=0.01))
	{
		float2 s = input[0].pos.xy+float2(0.5/g_diffX,-0.5/g_diffY);
		float2 dX = float2( 1.0,0);
		float2 dY = float2(0,1.0);
		p0.oPosition = float4( s+dX, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY+dX, 1.4142, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY+dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	return;
}


[maxvertexcount(4)]
void BlurDistGSBL( point VS_SIMPLE_INPUT input[1], inout TriangleStream<PS_INPUT> tStream )
{
	PS_INPUT p0;
	float bMask = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y), 0).w;
	float bMaskBL = g_inTex.SampleLevel(PointSampler, float2(input[0].pos.x,1-input[0].pos.y)+float2(-1.0/g_diffX,-1.0/g_diffY), 0).w;
	if ((bMask>0.01) && (bMaskBL<=0.01))
	{
		float2 s = input[0].pos.xy+float2(-0.5/g_diffX,0.5/g_diffY);
		float2 dX = float2(-1.0,0);
		float2 dY = float2(0, -1.0);
		p0.oPosition = float4( s+dX, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s, 0, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = float2(0,0);
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY+dX, 1.4142, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY+dX;
		tStream.Append( p0 );
		p0.oPosition = float4( s+dY, 1, 1.0);
		p0.oPosition.xy = p0.oPosition.xy*2-float2(1,1);
		p0.Tex = dY;
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	return;
}










[maxvertexcount(4)]
void CurveGS1( line VS_CURVE_INPUT input[2], inout TriangleStream<PS_CURVE_INPUT> tStream )
{
	PS_CURVE_INPUT p0;
	float2 nL = input[1].pos.xy-input[0].pos.xy;
	nL = normalize(float2(-nL.y, nL.x));

	// left side
	p0.oPosition = float4( input[0].pos.xy+g_polySize*nL, zMax, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[0].col0;
	p0.oCol = input[0].col1;
	p0.distdir = float4(0.5*g_polySize,input[0].pos.xy,0);
	tStream.Append( p0 );
	p0.oPosition = float4( input[1].pos.xy+g_polySize*nL, zMax, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[1].col0;
	p0.oCol = input[1].col1;
	p0.distdir = float4(0.5*g_polySize,input[1].pos.xy,0);
	tStream.Append( p0 );
	p0.oPosition = float4( input[0].pos.xy, 0.0, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[0].col0;
	p0.oCol = input[0].col1;
	p0.distdir = float4(0,input[0].pos.xy,0);
	tStream.Append( p0 );
	p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[1].col0;
	p0.oCol = input[1].col1;
	p0.distdir = float4(0,input[1].pos.xy,0);
	tStream.Append( p0 );
	tStream.RestartStrip();
}



[maxvertexcount(4)]
void CurveGS2( line VS_CURVE_INPUT input[2], inout TriangleStream<PS_CURVE_INPUT> tStream )
{
	PS_CURVE_INPUT p0;
	p0.pos = float2(0,0);
	float2 nL = input[1].pos.xy-input[0].pos.xy;
	nL = normalize(float2(-nL.y, nL.x));

	//right side
	p0.oPosition = float4( input[0].pos.xy, 0.0, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[0].col1;
	p0.oCol = input[0].col0;
	p0.distdir = float4(0,input[0].pos.xy,0);
	tStream.Append( p0 );
	p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[1].col1;
	p0.oCol = input[1].col0;
	p0.distdir = float4(0,input[1].pos.xy,0);
	tStream.Append( p0 );
	p0.oPosition = float4( input[0].pos.xy-g_polySize*nL, zMax, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[0].col1;
	p0.oCol = input[0].col0;
	p0.distdir = float4(0.5*g_polySize,input[0].pos.xy,0);
	tStream.Append( p0 );
	p0.oPosition = float4( input[1].pos.xy-g_polySize*nL, zMax, 1.0);
	p0.pos = p0.oPosition.xy;
	p0.col = input[1].col1;
	p0.oCol = input[1].col0;
	p0.distdir = float4(0.5*g_polySize,input[1].pos.xy,0);
	tStream.Append( p0 );
	tStream.RestartStrip();
}



[maxvertexcount(8)]
void CurveGS3( line VS_CURVE_INPUT input[2], inout TriangleStream<PS_CURVE_INPUT> tStream )
{
	// close the gaps to the following line
	if (input[1].nb.x < 9999.0)
	{
		PS_CURVE_INPUT p0;

		float2 nL = input[1].pos.xy-input[0].pos.xy;
		nL = normalize(float2(-nL.y, nL.x));
		float2 nLN = input[1].nb.xy-input[1].pos.xy;
		nLN = normalize(float2(-nLN.y, nLN.x));

		//larger than 90 degree angle -> form 4 polygons
		if (dot(input[0].pos.xy-input[1].pos.xy,input[1].nb.xy-input[1].pos.xy) >= 0)
		{
			float2 lP = -normalize(normalize(input[0].pos.xy-input[1].pos.xy) + normalize(input[1].nb.xy-input[1].pos.xy));
			float4 col = input[1].col0;
			float4 oCol = input[1].col1;
			float2 lH1,lH2;
			if (dot(nL, input[1].nb.xy-input[1].pos.xy) > 0)
			{
				col = input[1].col1;
				oCol = input[1].col0;
				nL = -nL;
				nLN = -nLN;
				lH1 = normalize(nL+lP);
				lH2 = normalize(nLN+lP);
			}
			else
			{
				lH1 = normalize(nL+lP);
				lH2 = normalize(nLN+lP);
			}
			p0.col = col;
			p0.oCol = oCol;
			
			p0.oPosition = float4( input[1].pos+g_polySize*nL, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos+g_polySize*lH1, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy+g_polySize*lP, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			tStream.RestartStrip();

			p0.oPosition = float4( input[1].pos.xy+g_polySize*lP, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,-g_polySize*lP,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos+g_polySize*lH2, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy+g_polySize*nLN, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			tStream.RestartStrip();
		}
		else if (dot(nL,nLN) <= 0.7071)	//between 45 and 90 degree angle -> form 2 polygons
		{
			float4 col = input[1].col0;
			float4 oCol = input[1].col1;
			if ( dot(input[1].nb.xy-input[1].pos.xy,nL) >= 0 )
			{
				col = input[1].col1;
				oCol = input[1].col0;
				nL = - nL;
				nLN = -nLN;
			}
			float2 lH = normalize(nL+nLN);
			p0.col = col;
			p0.oCol = oCol;
			p0.oPosition = float4( input[1].pos.xy+g_polySize*nL, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy+g_polySize*lH, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy+g_polySize*nLN, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			tStream.RestartStrip();
		}
		else //smaller than 45 degrees angle -> form 1 polygon
		{
			float4 col = input[1].col0;
			float4 oCol = input[1].col1;
			if ( dot(input[1].nb.xy-input[1].pos.xy,nL) >= 0 )
			{
				col = input[1].col1;
				oCol = input[1].col0;
				nL = - nL;
				nLN = -nLN;
			}
			p0.col = col;
			p0.oCol = oCol;
			p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy+g_polySize*nL, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			p0.oPosition = float4( input[1].pos.xy+g_polySize*nLN, zMax, 1.0);
			p0.pos = p0.oPosition.xy;
			p0.distdir = float4(0,input[1].pos.xy,1);
			tStream.Append( p0 );
			tStream.RestartStrip();
		}
	}		
}



[maxvertexcount(4)]
void CurveGS4( line VS_CURVE_INPUT input[2], inout TriangleStream<PS_CURVE_INPUT> tStream )
{
	if (input[0].nb.x > 9999.0)
	{
		// fill the beginning of the line if we are at a starting point
		PS_CURVE_INPUT p0;
		float2 nL = input[1].pos.xy-input[0].pos.xy;
		nL = normalize(float2(-nL.y, nL.x));
		float2 lP = -normalize(input[1].pos.xy-input[0].pos.xy);
		float2 lH = normalize(nL+lP);
	
		p0.col = input[0].col0;
		p0.oCol = input[0].col1;
		p0.oPosition = float4( input[0].pos.xy+g_polySize*nL, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[0].pos.xy, 0.0, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[0].pos.xy+g_polySize*lH, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[0].pos.xy+g_polySize*lP, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	else if (input[1].nb.x > 9999.0)
	{
		// fill the end of the line if we are at an end point
		PS_CURVE_INPUT p0;

		float2 nL = input[1].pos.xy-input[0].pos.xy;
		nL = normalize(float2(-nL.y, nL.x));
		float2 lP = normalize(input[1].pos.xy-input[0].pos.xy);
		float2 lH = normalize(nL+lP);

		p0.col = input[1].col0;
		p0.oCol = input[1].col1;
		p0.oPosition = float4( input[1].pos.xy+g_polySize*nL, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[1].pos.xy+g_polySize*lH, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[1].pos.xy+g_polySize*lP, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
}




[maxvertexcount(4)]
void CurveGS5( line VS_CURVE_INPUT input[2], inout TriangleStream<PS_CURVE_INPUT> tStream )
{
	if (input[0].nb.x > 9999.0)
	{
		// fill the beginning of the line if we are at a starting point
		PS_CURVE_INPUT p0;
		float2 nL = input[1].pos.xy-input[0].pos.xy;
		nL = normalize(float2(-nL.y, nL.x));
		float2 lP = -normalize(input[1].pos.xy-input[0].pos.xy);
		float2 lH = normalize(nL+lP);
	
		p0.col = input[0].col1;
		p0.oCol = input[0].col0;
		p0.oPosition = float4( input[0].pos.xy+g_polySize*lP, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[0].pos.xy, 0.0, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		lH = normalize(-nL+lP);
		p0.oPosition = float4( input[0].pos.xy+g_polySize*lH, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[0].pos.xy-g_polySize*nL, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[0].pos.xy,1);
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
	else if (input[1].nb.x > 9999.0)
	{
		// fill the end of the line if we are at an end point
		PS_CURVE_INPUT p0;

		float2 nL = input[1].pos.xy-input[0].pos.xy;
		nL = normalize(float2(-nL.y, nL.x));
		float2 lP = normalize(input[1].pos.xy-input[0].pos.xy);
		float2 lH = normalize(nL+lP);

		p0.col = input[1].col1;
		p0.oCol = input[1].col0;
		p0.oPosition = float4( input[1].pos.xy+g_polySize*lP, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[1].pos.xy, 0.0, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		lH = normalize(-nL+lP);
		p0.oPosition = float4( input[1].pos.xy+g_polySize*lH, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		p0.oPosition = float4( input[1].pos.xy-g_polySize*nL, zMax, 1.0);
		p0.pos = p0.oPosition.xy;
		p0.distdir = float4(0,input[1].pos.xy,1);
		tStream.Append( p0 );
		tStream.RestartStrip();
	}
}





// ================================================================================
//
// The Pixel Shaders
//
// ================================================================================


// write distance and direction info from the curves
PS_OUTPUT CurveDistPS(PS_CURVE_INPUT In)
{
	PS_OUTPUT Out; 
	float dist = length(In.pos.xy-In.distdir.yz);
	Out.oColor = float4( 0.5*dist, In.pos.xy-In.distdir.yz, 0.00000001*dist);  // the "structure" texture get (distance to boundary, XX, blur value, distance weight)
	Out.oColor.z = -Out.oColor.z;
	return Out;
}


// write curve color info from the curves
PS_OUTPUT CurveColorPS(PS_CURVE_INPUT In)
{
	PS_OUTPUT Out; 
	Out.oColor = In.col;
	return Out;
}


// write distance 
PS_OUTPUT BlurDistPS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	float dist = length(In.Tex.xy);
	Out.oColor = float4(dist,dist,dist,dist); 
	return Out;
}



// diffuse the distance as biharmonic/bilaplace function to get the boundary distace function W_l
PS_OUTPUT DiffuseDistPoissonPS(PS_INPUT In)
{
    PS_OUTPUT Out; 
	Out.oColor.xyz = g_inTex.SampleLevel(PointSampler, In.Tex.xy, 0).xyz;
	float rawKernel = 0.92387*g_inTex.SampleLevel(PointSampler, In.Tex.xy, 0).x;
	float kernelX = max(0, rawKernel*g_diffX*g_polySize-1.0)/g_diffX;
	float kernelY = max(0, rawKernel*g_diffY*g_polySize-1.0)/g_diffY;
	Out.oColor.w =  g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(-kernelX,0), 0).w;
	Out.oColor.w += g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(+kernelX,0), 0).w;
	Out.oColor.w += g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0,-kernelY), 0).w;
	Out.oColor.w += g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0,+kernelY), 0).w;
	Out.oColor.w *= 0.25;
	Out.oColor.w += 2.0*kernelX*kernelY; // make Laplacian scale invariant to kernel size
	return Out;
}




// compute the smooth distance component (indicating the layer number later) from the biharmonic function
PS_OUTPUT GetSmoothDistFromPoissonPS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	Out.oColor = g_inTex.Sample(PointSampler, In.Tex.xy).xyzw;
	if (0.5*max(abs(Out.oColor.y)*g_diffX, abs(Out.oColor.z)*g_diffY) < 0.71)
		Out.oColor.w = 0.0;
	else
	{
		// cosine between circular inflated function 3D normal and vector float3(0,0,1)
		float3 gDX = g_diffX*(sqrt(g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(+0.5/g_diffX,0), 0).w)-sqrt(g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(-0.5/g_diffX,0), 0).w));
		float3 gDY = g_diffY*(sqrt(g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0,+0.5/g_diffY), 0).w)-sqrt(g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0,-0.5/g_diffY), 0).w));
		Out.oColor.w = 1.0/sqrt(1+gDX.z*gDX.z+gDY.z*gDY.z);
	}
	return Out;
}





PS_OUTPUT DiffuseColorsPS(PS_INPUT In)
{
    PS_OUTPUT Out; 
	// diffuse the colors as harmonic/Laplace function
	float rawKernel = 0.92387*g_distDirTex.SampleLevel(PointSampler, In.Tex.xy, 0).x;
	float kernelX = max(0,rawKernel-1.0/g_diffX)*g_polySize;
	float kernelY = max(0,rawKernel-1.0/g_diffY)*g_polySize;
	Out.oColor =  g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(-kernelX,0), 0);
	Out.oColor += g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2( kernelX,0), 0);
	Out.oColor += g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0,-kernelY), 0);
	Out.oColor += g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0, kernelY), 0);
	Out.oColor *= 0.25;
	// inhomogenuous blur value diffusion at the second pass
	if (g_showLayers > 0.5)  
	{
		float w1 = 1-g_distDirTex.SampleLevel(LinearSampler, In.Tex.xy+0.5*float2(-kernelX,0), 0).w;
		float w2 = 1-g_distDirTex.SampleLevel(LinearSampler, In.Tex.xy+0.5*float2( kernelX,0), 0).w;
		float w3 = 1-g_distDirTex.SampleLevel(LinearSampler, In.Tex.xy+0.5*float2(0,-kernelY), 0).w;
		float w4 = 1-g_distDirTex.SampleLevel(LinearSampler, In.Tex.xy+0.5*float2(0, kernelY), 0).w;
		Out.oColor.w =  w1*g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(-kernelX,0), 0).w;
		Out.oColor.w += w2*g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2( kernelX,0), 0).w;
		Out.oColor.w += w3*g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0,-kernelY), 0).w;
		Out.oColor.w += w4*g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0, kernelY), 0).w;
		Out.oColor.w /= (w1+w2+w3+w4);
	}
	return Out;
}




PS_OUTPUT AddToOutputPS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	if (g_showLayers < 0.5)  
		Out.oColor.xyzw = g_inTex.Sample(PointSampler, In.Tex.xy).xyzw;   // first layer
	else	// second/higher layer
	{
		// color map
		float f = g_distDirTex.SampleLevel(PointSampler, In.Tex.xy, 0).w;
		Out.oColor.xyz = (1-f)*g_inTex2.Sample(PointSampler, In.Tex.xy).xyz + f*g_inTex.Sample(PointSampler, In.Tex.xy).xyz;
		// inhomogenuously diffused blur map 
		if (g_inTex.Sample(PointSampler, In.Tex.xy).w == 0)
			Out.oColor.w = 1;
		else
			Out.oColor.w = min(1, g_distDirTex.Sample(PointSampler, In.Tex.xy).w/g_inTex.Sample(PointSampler, In.Tex.xy).w);
		Out.oColor.w = (0.5+0.5*cos(Out.oColor.w*M_PI));
	}
	return Out;
}




PS_OUTPUT BlurBoundaryPS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	Out.oColor = g_inTex.SampleLevel(PointSampler, In.Tex.xy, 0);
	float w = Out.oColor.w;
	if (w > 0.0)
	{
		// diffuse the colors as harmonic/Laplace function while fitting the original function
		float rawKernel = 0.92387*g_distDirTex.SampleLevel(PointSampler, In.Tex.xy, 0).x;
		float kernel = rawKernel*g_polySize;
		float3 fdx = g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(kernel,0), 0).xyz;
		float3 fdy = g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0, kernel), 0).xyz;
		float3 fdx2 = g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(-kernel,0), 0).xyz;
		float3 fdy2 = g_inTex.SampleLevel(LinearSampler, In.Tex.xy+float2(0, -kernel), 0).xyz;
		float3 a = g_inTex2.SampleLevel(PointSampler, In.Tex.xy, 0).xyz;
		float3 b = fdx+fdy+fdx2+fdy2;
		Out.oColor.xyz = (1-w)*a+w*0.25*b;
	}
	return Out;
}




PS_OUTPUT AntiAliasPS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	Out.oColor = g_inTex.SampleLevel(PointSampler, In.Tex.xy, 0);
	float4 distDir = g_distDirTex.SampleLevel(PointSampler, In.Tex.xy, 0);
	if ((Out.oColor.w <= 0.001) && (distDir.x*g_diffX < 0.7))
	{
		// diffuse the colors as harmonic/Laplace function while fitting the original function
		float2 dd = -distDir.yz;
		dd.xy = normalize(dd.xy);
		dd.y = dd.y;
		float3 thisColor = Out.oColor.xyz;
		float3 otherColor = g_inTex.SampleLevel(PointSampler, In.Tex.xy+1.4*dd*float2(1.0/g_diffX, 1.0/g_diffY), 0).xyz;
		Out.oColor.xyz = (1.0-(distDir.x*g_diffX/0.7))*0.5*(otherColor+thisColor) + (distDir.x*g_diffX/0.7)*thisColor;
	}
	return Out;
}



PS_OUTPUT DisplayImage(PS_INPUT In)
{
	PS_OUTPUT Out; 
	Out.oColor = g_inTex.Sample(PointSampler, In.Tex.xy).xyzw;
	if (g_showLayers == 1)
		Out.oColor.xyz = g_inTex.SampleLevel(PointSampler, In.Tex.xy, 0).www;
	if (g_showLayers == 2)
		Out.oColor.xyz = 1-g_inTex.SampleLevel(PointSampler, In.Tex.xy, 0).www;
	if (g_showLayers == 3)
		Out.oColor.xyz = 0.1+20*g_inTex.SampleLevel(PointSampler, In.Tex.xy, 0).w;
	return Out;
}





PS_OUTPUT DrawOverlayPS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	Out.oColor = float4(0,0,0,1);
	return Out;
}




PS_OUTPUT ControlPointPS(PS_CONTROLPOINTINPUT In)
{
	PS_OUTPUT Out; 
	float dist = In.tex.x*In.tex.x+In.tex.y*In.tex.y;
	if (dist > 1.0)
		clip(-1);
	float3 col = 0;
	if (dist < 0.6)
	{
		col = In.col;
		if (In.col.x < -9999)
			clip(-1);
	}
	Out.oColor = float4(col,1); 
	return Out;
}



PS_OUTPUT PalettePS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	if (cmpColor == 0)  // color palette
	{
		Out.oColor = g_paletteTex.Sample(PointSampler, In.Tex.xy);
		{
			float3 col = g_mousePointerTex.SampleLevel(LinearSampler, float2(2.0,4.0)*(In.Tex.xy-g_pan.xy) +(In.Tex.xy+float2(0.51,0.5)-g_pan.xy),0);
			if (col.x > 0.01)
				Out.oColor.xyz = col;
		}
	}
	if (cmpColor == 1)  // lightness palette
	{
		Out.oColor = float4( 1-In.Tex.y, 1-In.Tex.y, 1-In.Tex.y, 1);
		if (abs(In.Tex.y-g_pan.y) < 2.0/256.0)
			Out.oColor = float4(1,0,0,0);
	}
	return Out;
}



PS_OUTPUT MousePointerPS(PS_INPUT In)
{
	PS_OUTPUT Out; 
	Out.oColor.xyz = g_mousePointerTex.SampleLevel(LinearSampler, 15*(float2(0.5*In.Tex.x,In.Tex.y)-g_pan.xy) +(In.Tex.xy+float2(0.51,0.5)-g_pan.xy),0);
	if (Out.oColor.x < 0.01)
		clip(-1);
	Out.oColor.w = 1.0-Out.oColor.x;
	return Out;
}



//--------------------------------------------------------------------------------------
// Techniques/states
//--------------------------------------------------------------------------------------

RasterizerState state
{
	FillMode = Solid;
	CullMode = none;
	MultisampleEnable = FALSE;
};


RasterizerState stateOutline
{
	FillMode = WireFrame;
	CullMode = none;
	MultisampleEnable = FALSE;
};


DepthStencilState soliddepth
{
	DepthEnable = FALSE;
};


DepthStencilState depthEnable
{
	DepthEnable = TRUE;
	Depthfunc = LESS_EQUAL;
	DepthWriteMask = ALL;
	StencilEnable = FALSE;
};


DepthStencilState depthEnablePassive
{
	DepthEnable = TRUE;
	Depthfunc = LESS_EQUAL;
	DepthWriteMask = ZERO;
	StencilEnable = FALSE;
};


BlendState NoBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = FALSE;
};

BlendState BlendAdd
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
	SrcBlend = ONE;
	DestBlend = ZERO;
  	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ONE;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[0] = 0x0F;
};



technique10 DrawCurvesDistance
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS1( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P2
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS2( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P3
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS3( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P4
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS4( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }         
	pass P5
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS5( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }         
}


technique10 DrawCurvesColors
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS1( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveColorPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnablePassive, 0 );
    }
	pass P2
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS2( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveColorPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnablePassive, 0 );
    }
	pass P3
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS3( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveColorPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnablePassive, 0 );
    }
	pass P4
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS4( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveColorPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnablePassive, 0 );
    }         
	pass P5
	{
        SetVertexShader( CompileShader( vs_4_0, CurveVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, CurveGS5( ) ) );
        SetPixelShader( CompileShader( ps_4_0, CurveColorPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnablePassive, 0 );
    }         
}




technique10 DiffuseDistance
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DiffuseDistPoissonPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}


technique10 GetSmoothDistFromPoisson
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, GetSmoothDistFromPoissonPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}



technique10 BlurrDistance
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, BlurDistVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, BlurDistGSCenter( ) ) );
        SetPixelShader( CompileShader( ps_4_0, BlurDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P2
	{
        SetVertexShader( CompileShader( vs_4_0, BlurDistVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, BlurDistGSLeftRight( ) ) );
        SetPixelShader( CompileShader( ps_4_0, BlurDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P3
	{
        SetVertexShader( CompileShader( vs_4_0, BlurDistVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, BlurDistGSTopbottom( ) ) );
        SetPixelShader( CompileShader( ps_4_0, BlurDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P4
	{
        SetVertexShader( CompileShader( vs_4_0, BlurDistVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, BlurDistGSTL( ) ) );
        SetPixelShader( CompileShader( ps_4_0, BlurDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P5
	{
        SetVertexShader( CompileShader( vs_4_0, BlurDistVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, BlurDistGSBR( ) ) );
        SetPixelShader( CompileShader( ps_4_0, BlurDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P6
	{
        SetVertexShader( CompileShader( vs_4_0, BlurDistVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, BlurDistGSTR( ) ) );
        SetPixelShader( CompileShader( ps_4_0, BlurDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
	pass P7
	{
        SetVertexShader( CompileShader( vs_4_0, BlurDistVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, BlurDistGSBL( ) ) );
        SetPixelShader( CompileShader( ps_4_0, BlurDistPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( depthEnable, 0 );
    }
}




technique10 DiffuseColors
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DiffuseColorsPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}



technique10 AddToOutput
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, AddToOutputPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}




technique10 BlurDiffuseColors
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, BlurBoundaryPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}


technique10 AntiAlias
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, AntiAliasPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}



technique10 DisplayOutput
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DisplayImage( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}



technique10 DrawCurvesOverlay
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, CurveOverlayVS( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DrawOverlayPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}



technique10 ShowControlPoint
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ControlPointVS( ) ) );
        SetGeometryShader( CompileShader( gs_4_0, ControlPointGS( ) ) );
        SetPixelShader( CompileShader( ps_4_0, ControlPointPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}



technique10 DisplayPalette
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PalettePS( ) ) );
        SetRasterizerState( state );
        SetBlendState( NoBlending, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}


technique10 DisplayMouse
{
	pass P1
	{
        SetVertexShader( CompileShader( vs_4_0, ScreenQuad( ) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, MousePointerPS( ) ) );
        SetRasterizerState( state );
        SetBlendState( BlendAdd, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
        SetDepthStencilState( soliddepth, 0 );
    }
}
