// Include the OS headers
//-----------------------
#include <windows.h>
#include <atlbase.h>
#pragma warning( disable: 4996 )
#include <strsafe.h>
#pragma warning( default: 4996 )
// Include the D3D10 headers
//--------------------------
#include <d3d10.h>
#include <d3dx10.h>
#include <d3dx9.h>

#include <string>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>

#include "DXUT.h"
#include "DXUTgui.h"
#include "DXUTsettingsDlg.h"
#include "SDKmisc.h"
#include "SDKMesh.h"
#include "VSObject.h"
#include "Globals.h"

#define M_PI 6.2831853


D3D10_INPUT_ELEMENT_DESC VSObject::InputElements[] =
{
    { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ NULL, 0, DXGI_FORMAT_UNKNOWN, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 }
};
int VSObject::InputElementCount = 2;


D3D10_INPUT_ELEMENT_DESC VSObject::InputCurveElements[] =
{
    { "POSITION", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 0, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 8, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 1, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 2, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 40, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 3, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 56, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "TEXTURE", 4, DXGI_FORMAT_R32G32_FLOAT, 0, 72, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 5, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 80, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 6, DXGI_FORMAT_R32G32B32A32_FLOAT, 0, 96, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "TEXTURE", 7, DXGI_FORMAT_R32G32_FLOAT, 0, 112, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ NULL, 0, DXGI_FORMAT_UNKNOWN, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 }
};
int VSObject::InputCurveElementCount = 9;


D3D10_INPUT_ELEMENT_DESC VSObject::ScreenSpaceElements[] =
{
    { "POSITION", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ NULL, 0, DXGI_FORMAT_UNKNOWN, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 }
};
int VSObject::ScreenSpaceElementCount = 1;


D3D10_INPUT_ELEMENT_DESC VSObject::ControlPointElementLayout[] =
{
    { "POSITION", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXTURE", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 8, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ NULL, 0, DXGI_FORMAT_UNKNOWN, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 }
};
int VSObject::InputControlPointElementCount = 2;




// computes the distance between a point and a line segment
double PointLineSegDist(double pX, double pY, double l1X, double l1Y, double l2X, double l2Y)
{
      double vX = l2X - l1X;
      double vY = l2Y - l1Y;
      double wX = pX - l1X;
      double wY = pY - l1Y;
	  double c1 = wX*vX+wY*vY;
      if (c1<=0.0)
            return ((pX-l1X)*(pX-l1X)+(pY-l1Y)*(pY-l1Y));
	  double c2 = vX*vX+vY*vY;
      if (c2 <= c1)
            return ((pX-l2X)*(pX-l2X)+(pY-l2Y)*(pY-l2Y));
      double b = c1 / c2;
      double PbX = l1X + b*vX;
      double PbY = l1Y + b*vY;
      return ((pX-PbX)*(pX-PbX)+(pY-PbY)*(pY-PbY));
}



int VSObject::GetNearestSegmentID( float pX, float pY, int &id1, float &weight, int &idPos, float &wPos, int &idBlur, float &wBlur)
{
	double dist = 1000000000.0;
	int ID = -1;
	for (int i=0; i<m_cSegNum*subSegNum*10; i++)
	{
		double d = PointLineSegDist(pX, pY, pd[i*2+0].pos.x, pd[i*2+0].pos.y, pd[i*2+1].pos.x, pd[i*2+1].pos.y);
		if (d < dist)
		{
			dist = d;
			ID = i;
		}
	}
	idPos = pd[ID*2].IDwPos.x;
	wPos = pd[ID*2].IDwPos.y;
	D3DXVECTOR3 v0 = D3DXVECTOR3(pd[ID*2+1].pos.x-pd[ID*2+0].pos.x, pd[ID*2+1].pos.y-pd[ID*2+0].pos.y, 0.0);
	D3DXVECTOR3 v1 = D3DXVECTOR3(pX-pd[ID*2+0].pos.x, pY-pd[ID*2+0].pos.y, 0.0);
	D3DXVECTOR3 v2;
	D3DXVec3Cross(&v2, &v0, &v1);
	if(v2.z>0)
	{
		id1 = pd[ID*2].IDw.x;
		weight = pd[ID*2].IDw.y;
		idBlur = pd[ID*2].IDwB.x;
		wBlur = pd[ID*2].IDwB.y;
		if (pd[ID*2].IDw.y > 0.5)	// this is the return value, which is only used by the graphic interface
			ID = pd[ID*2].IDw.x;
		else
			ID = pd[ID*2].IDw.x+1;
	}
	else
	{
		id1 = pd[ID*2].IDw.z;
		weight = pd[ID*2].IDw.w;
		idBlur = pd[ID*2].IDwB.z;
		wBlur = pd[ID*2].IDwB.w;
		if (pd[ID*2].IDw.w > 0.5)  // this is the return value, which is only used by the graphic interface
			ID = pd[ID*2].IDw.z;
		else
			ID = pd[ID*2].IDw.z+1;
	}	
	return ID;
}





VSObject::VSObject(ID3D10Device *pd3dDevice)
{
	m_pVertexBuffer = NULL;
	m_pVertexLayout = NULL;
	m_pDepthStencil = NULL;
	m_diffuseTexture[0] = NULL;
	m_diffuseTexture[1] = NULL;
	m_distDirTexture = NULL;
	m_otherTexture = NULL;
	m_fitCPUTexture = NULL;
	m_paletteTextureRV = NULL;
	m_mousePointerTextureRV = NULL;
	m_pMeshControlPoints = NULL;
	m_controlPointData = NULL;
	diffTex = 0;
	diffSteps = 32;
	diffSteps2 = 32;
	diffSteps3 = 32;
	m_pMeshDiff = NULL;
	m_pMeshCurves = NULL;
	m_cNum = 0;
	m_curve = NULL;
	m_scale = 1.0f;
	m_1L = 0.0f;
	m_2L = 0.0f;
	m_1R = 0.0f;
	m_2R = 0.0f;
	m_blurScale = 1.0f;
	m_pan = D3DXVECTOR2(0,0);
	m_polySize = 1.0;
	pd = NULL;
	m_wImage[0] = NULL;
	m_inputI = NULL;
	// create output quad
	m_sizeX = ( DXUTIsAppRenderingWithD3D9() ) ? DXUTGetD3D9BackBufferSurfaceDesc()->Width : DXUTGetDXGIBackBufferSurfaceDesc()->Width;
	m_sizeY = ( DXUTIsAppRenderingWithD3D9() ) ? DXUTGetD3D9BackBufferSurfaceDesc()->Height : DXUTGetDXGIBackBufferSurfaceDesc()->Height;
}


VSObject::~VSObject(void)
{
//	SAFE_RELEASE( pMesh );
	delete[](m_cPID);
	delete[](m_cPID2);
	delete[](m_cPBID);
	delete[](m_cSplineID);
	delete[](pd);
	for (int i=0; i<maxWNum; i++)
	{
		delete[](m_wImage[i]);
		delete[](m_wImage2[i]);
	}
	delete m_inputI;
}




bool stringStartsWith(const char *s, const char *val)
{
        return !strncmp(s, val, strlen(val));
}




void VSObject::SetupTextures(ID3D10Device *pd3dDevice, ID3D10Effect* g_pEffect10, int sizeX, int sizeY)
{
	HRESULT hr;

	// create the new mesh
	if (m_pMeshDiff == NULL)
	{
		//create the screen space quad
		D3DXVECTOR3 pos[3*2] = {	D3DXVECTOR3(-1.0f,-1.0f,+0.5f), D3DXVECTOR3( 1.0f,-1.0f,+0.5f), D3DXVECTOR3(-1.0f,1.0f,+0.5f),
									D3DXVECTOR3( 1.0f,-1.0f,+0.5f), D3DXVECTOR3( 1.0f, 1.0f,+0.5f), D3DXVECTOR3(-1.0f,1.0f,+0.5f) };
		D3DXVECTOR2 tex[3*2] = {	D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 0.0f), D3DXVECTOR2(0.0f, 1.0f),
									D3DXVECTOR2(1.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f), D3DXVECTOR2(0.0f, 1.0f)};
		VSO_Vertex pd[2*3];
		for (int ctri=0; ctri<2; ctri++)
		{
			UINT32 i = 3*ctri;
			pd[i+0].pos = pos[i+0];
			pd[i+1].pos = pos[i+1];
			pd[i+2].pos = pos[i+2];
			pd[i+0].tex = tex[i+0];
			pd[i+1].tex = tex[i+1];
			pd[i+2].tex = tex[i+2];
		}
		V( D3DX10CreateMesh(pd3dDevice, VSObject::InputElements, VSObject::InputElementCount, "POSITION", 2*3, 2, 0, &m_pMeshDiff) );
		V( m_pMeshDiff->SetVertexData(0, pd) );
		V( m_pMeshDiff->CommitToDevice() );
	}

	m_sizeX = sizeX;
	m_sizeY = sizeY;
	m_vp.Width = (int)(m_sizeX);
	m_vp.Height = (int)(m_sizeY);
	m_vp.MinDepth = 0.0f;
	m_vp.MaxDepth = 1.0f;
	m_vp.TopLeftX = 0;
	m_vp.TopLeftY = 0;

	// connect to shader variables
	m_pDrawCurveDistancesTechnique = g_pEffect10->GetTechniqueByName( "DrawCurvesDistance" );
	m_pDrawCurveColorsTechnique = g_pEffect10->GetTechniqueByName( "DrawCurvesColors" );
	m_pDiffuseDistancesTechnique = g_pEffect10->GetTechniqueByName( "DiffuseDistance" );
	m_pDistToYTechnique = g_pEffect10->GetTechniqueByName( "GetSmoothDistFromPoisson" );
	m_pAddToOutputTechnique = g_pEffect10->GetTechniqueByName( "AddToOutput" );
	m_pDrawControlPointTechnique = g_pEffect10->GetTechniqueByName( "ShowControlPoint" );

	m_pDiffuseColorsTechnique = g_pEffect10->GetTechniqueByName( "DiffuseColors" );
	m_pBlurTechnique = g_pEffect10->GetTechniqueByName( "BlurDiffuseColors" );
	m_pBlurrDistTechnique = g_pEffect10->GetTechniqueByName( "BlurrDistance" );
	m_pAntiAliasTechnique = g_pEffect10->GetTechniqueByName( "AntiAlias" );
	m_pDisplayImage = g_pEffect10->GetTechniqueByName( "DisplayOutput" );
	m_pDrawCurvesOverlay = g_pEffect10->GetTechniqueByName( "DrawCurvesOverlay" );
	m_pDisplayPalette = g_pEffect10->GetTechniqueByName( "DisplayPalette" );
	m_pDisplayMouse = g_pEffect10->GetTechniqueByName( "DisplayMouse" );

	m_pDistDirTex = g_pEffect10->GetVariableByName( "g_distDirTex" )->AsShaderResource();
	m_pInTex = g_pEffect10->GetVariableByName( "g_inTex" )->AsShaderResource();
	m_pInTex2 = g_pEffect10->GetVariableByName( "g_inTex2" )->AsShaderResource();
	m_pPaletteTex = g_pEffect10->GetVariableByName( "g_paletteTex" )->AsShaderResource();
	m_pMousePointerTex = g_pEffect10->GetVariableByName( "g_mousePointerTex" )->AsShaderResource();
	m_pDiffX = g_pEffect10->GetVariableByName( "g_diffX")->AsScalar();
	m_pDiffY = g_pEffect10->GetVariableByName( "g_diffY")->AsScalar();
	g_pScale = g_pEffect10->GetVariableByName( "g_scale")->AsScalar();
	g_pPan = g_pEffect10->GetVariableByName( "g_pan")->AsVector();
	m_pPolySize = g_pEffect10->GetVariableByName( "g_polySize")->AsScalar();
	m_pShowLayers = g_pEffect10->GetVariableByName( "g_showLayers")->AsScalar();
	m_pBlurScale = g_pEffect10->GetVariableByName( "g_blurScale")->AsScalar();
	m_pColLScale = g_pEffect10->GetVariableByName( "g_colLScale")->AsScalar();
	m_pColRScale = g_pEffect10->GetVariableByName( "g_colRScale")->AsScalar();
	m_pCmpColor = g_pEffect10->GetVariableByName( "cmpColor")->AsScalar();


	D3D10_TEXTURE2D_DESC texdesc;
	ID3D10Resource *pTexture;
	D3D10_MAPPED_TEXTURE2D pMappedTex;
	D3DX10_IMAGE_LOAD_INFO l;
	l.Width = D3DX10_DEFAULT;
	l.Height = D3DX10_DEFAULT;
	l.Depth = D3DX10_DEFAULT;
	l.FirstMipLevel = D3DX10_DEFAULT;
	l.MipLevels = D3DX10_DEFAULT;
	l.Usage = D3D10_USAGE_STAGING;
	l.BindFlags = 0;
	l.CpuAccessFlags = D3D10_CPU_ACCESS_READ;
	l.MiscFlags = D3DX10_DEFAULT;
	l.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	l.Filter = D3DX10_DEFAULT;
	l.MipFilter = D3DX10_DEFAULT;
	l.pSrcInfo = NULL;
	if (m_paletteTextureRV == NULL)
	{
		// read the palette image and create a copy in main memory
		V( D3DX10CreateShaderResourceViewFromFile(pd3dDevice, L"Media\\ColorPicker3.bmp", NULL, NULL, &m_paletteTextureRV, NULL) );
		V( D3DX10CreateTextureFromFile( pd3dDevice, L"Media\\ColorPicker3.bmp", &l, NULL, &pTexture, NULL) );
		( m_pPaletteTex->SetResource( m_paletteTextureRV ) );
		((ID3D10Texture2D*)(pTexture))->GetDesc( &texdesc );
		m_pPaletteImage = new float[256*256*4];
		V( ((ID3D10Texture2D*)(pTexture))->Map( 0, D3D10_MAP_READ, 0, &pMappedTex) );
		float *pData = (float*)(pMappedTex.pData);
		for (int i=0; i<256*256*4; i++)
			m_pPaletteImage[i] = pData[i];
		((ID3D10Texture2D*)(pTexture))->Unmap( 0 );
	}
	if (m_mousePointerTextureRV == NULL)  // read the mouse pointer image and link to shader variable
	{
		V( D3DX10CreateShaderResourceViewFromFile(pd3dDevice, L"Media\\MousePointer.bmp", NULL, NULL, &m_mousePointerTextureRV, NULL) );
		( m_pMousePointerTex->SetResource( m_mousePointerTextureRV ) );
	}
	// create the depth buffer (only needed for curve mask rendering)
	DXGI_SAMPLE_DESC samdesc;
	samdesc.Count = 1;
	samdesc.Quality = 0;
	ZeroMemory( &texdesc, sizeof(D3D10_TEXTURE2D_DESC) );
	texdesc.MipLevels = 1;
	texdesc.ArraySize = 1;
	texdesc.SampleDesc = samdesc;
	texdesc.Width = (int)(m_sizeX);
	texdesc.Height = (int)(m_sizeY);
	texdesc.Usage = D3D10_USAGE_DEFAULT;
	texdesc.BindFlags = D3D10_BIND_DEPTH_STENCIL;
	texdesc.Format = DXGI_FORMAT_D32_FLOAT;
	if (m_pDepthStencil != NULL)
		m_pDepthStencil->Release();
	hr = pd3dDevice->CreateTexture2D( &texdesc, NULL, &m_pDepthStencil );
	hr = pd3dDevice->CreateDepthStencilView( m_pDepthStencil, NULL, &m_pDepthStencilView );

	//create diffusion textures (2 for ping pong rendering)
	if (m_diffuseTexture[0] != NULL)
		m_diffuseTexture[0]->Release();
	if (m_diffuseTexture[1] != NULL)
		m_diffuseTexture[1]->Release();
	if (m_distDirTexture != NULL)
		m_distDirTexture->Release();
	if (m_otherTexture != NULL)
		m_otherTexture->Release();
	texdesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;  // use this for higher accuracy diffusion
	texdesc.BindFlags =  D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
	hr = pd3dDevice->CreateTexture2D( &texdesc, NULL, &m_diffuseTexture[0]);
	hr = pd3dDevice->CreateTexture2D( &texdesc, NULL, &m_diffuseTexture[1]);
	hr = pd3dDevice->CreateTexture2D( &texdesc, NULL, &m_resultTexture);
	hr = pd3dDevice->CreateTexture2D( &texdesc, NULL, &m_otherTexture);
	hr = pd3dDevice->CreateTexture2D( &texdesc, NULL, &m_distDirTexture);

	// CPU read texture, used for image fitting
	texdesc.Usage = D3D10_USAGE_STAGING;
	texdesc.BindFlags = 0; 
	texdesc.CPUAccessFlags = D3D10_CPU_ACCESS_READ;
	if (m_fitCPUTexture != NULL)
		m_fitCPUTexture->Release();
	hr = pd3dDevice->CreateTexture2D( &texdesc, NULL, &m_fitCPUTexture);

	//create render target views
	hr = pd3dDevice->CreateShaderResourceView( m_diffuseTexture[0], NULL, &m_diffuseTextureRV[0] );
	hr = pd3dDevice->CreateRenderTargetView(   m_diffuseTexture[0], NULL, &m_diffuseTextureTV[0] );
	hr = pd3dDevice->CreateShaderResourceView( m_diffuseTexture[1], NULL, &m_diffuseTextureRV[1] );
	hr = pd3dDevice->CreateRenderTargetView(   m_diffuseTexture[1], NULL, &m_diffuseTextureTV[1] );
	hr = pd3dDevice->CreateShaderResourceView( m_distDirTexture, NULL, &m_distDirTextureRV );
	hr = pd3dDevice->CreateRenderTargetView(   m_distDirTexture, NULL, &m_distDirTextureTV );
	hr = pd3dDevice->CreateShaderResourceView( m_resultTexture, NULL, &m_resultTextureRV );
	hr = pd3dDevice->CreateRenderTargetView(   m_resultTexture, NULL, &m_resultTextureTV );
	hr = pd3dDevice->CreateShaderResourceView( m_otherTexture, NULL, &m_otherTextureRV );
	hr = pd3dDevice->CreateRenderTargetView(   m_otherTexture, NULL, &m_otherTextureTV );

	m_pDisplayMesh = NULL;
	UpdateDisplayMesh(pd3dDevice);

	// generate a control point vertex buffer
	const int numControlPoints = 20000;
	if (m_pMeshControlPoints != NULL)
		m_pMeshControlPoints->Release();
	V( D3DX10CreateMesh(pd3dDevice, ControlPointElementLayout, InputControlPointElementCount, "POSITION", numControlPoints, numControlPoints, 0, &m_pMeshControlPoints) );
	if (m_controlPointData != NULL)
		delete[]m_controlPointData;
	m_controlPointData = new CONTROLPOINT_Vertex[numControlPoints];

	// get CPU memory for the weighting images
	if (m_wImage[0] != NULL)
		for (int i=0; i<maxWNum; i++)
		{
			delete[](m_wImage[i]);
			delete[](m_wImage2[i]);
		}
	for (int i=0; i<maxWNum; i++)
	{
		m_wImage[i] = new D3DXVECTOR2[m_sizeX*m_sizeY];
		m_wImage2[i] = new D3DXVECTOR2[m_sizeX*m_sizeY];
	}
}




void VSObject::UpdateDisplayMesh(ID3D10Device *pd3dDevice)
{
	HRESULT hr;
	int width = DXUTGetDXGIBackBufferSurfaceDesc()->Width;
	int height = DXUTGetDXGIBackBufferSurfaceDesc()->Height;
	if (m_pDisplayMesh != NULL)
		m_pDisplayMesh->Release();
	V( D3DX10CreateMesh(pd3dDevice, VSObject::ScreenSpaceElements, VSObject::ScreenSpaceElementCount, "POSITION", width*height, width*height, 0, &m_pDisplayMesh) );
	SIMPLE_Vertex *dpd = new SIMPLE_Vertex[width*height];
	for (int y=0; y<height; y++)
		for (int x=0; x<width; x++)
		{
			float xf = (float)(x)+0.5f;
			float yf = (float)(y)+0.5f;
			dpd[y*width+x].pos = D3DXVECTOR2(xf/width,yf/height);
		}
	V( m_pDisplayMesh->SetVertexData(0, dpd) );
	V( m_pDisplayMesh->CommitToDevice() );
	delete[](dpd);
}


// this writes new file format 
void VSObject::SaveGDCI(char *destColFile)
{
	// save the curves to a vector file in xml style of [Orzan et al.].  
	FILE *F = fopen( destColFile, "wb");
	fprintf( F, "<!DOCTYPE GDCICurveSetXML>\n");
	fprintf( F, "<curve_set image_width=\"%d\" image_height=\"%d\" nb_curves=\"%d\">\n", (int)(m_fWidth), (int)(m_fHeight), m_cNum);
	for (int i=0; i<m_cNum; i++)
	{
		fprintf( F, " <curve nb_control_points=\"%d\" nb_left_colors=\"%d\" nb_right_colors=\"%d\" nb_left_blur_points=\"%d\" nb_right_blur_points=\"%d\"> \n", m_curve[i].pNum, m_curve[i].clNum, m_curve[i].crNum, m_curve[i].bLNum, m_curve[i].bRNum);
		fprintf( F, " <control_points_set>\n");
		for (int i1=0; i1<m_curve[i].pNum; i1++)
		{
			float x = 0.5*(m_curve[i].p[i1].x+1.0)*m_fWidth;
			float y = 0.5*(m_curve[i].p[i1].y+1.0)*m_fHeight;
			fprintf( F, "   <control_point x=\"%f\" y=\"%f\" />\n", x, y);
		}
		fprintf( F, "  </control_points_set>\n");

		fprintf( F, "  <left_colors_set>\n");
		for (int i1=0; i1<m_curve[i].clNum; i1++)
		{
			fprintf( F, "   <left_color G=\"%d %d\" R=\"%d %d\" globalID=\"%d\" B=\"%d %d\" />\n", (int)(256.0f*m_curve[i].cl[i1].col.y), (int)(256.0f*m_curve[i].cl[i1].col2.y), (int)(256.0f*m_curve[i].cl[i1].col.z), (int)(256.0f*m_curve[i].cl[i1].col2.z), m_curve[i].cl[i1].off, (int)(256.0f*m_curve[i].cl[i1].col.x), (int)(256.0f*m_curve[i].cl[i1].col2.x));
		}
		fprintf( F, "  </left_colors_set>\n");

		fprintf( F, "  <right_colors_set>\n");
		for (int i1=0; i1<m_curve[i].crNum; i1++)
		{
			fprintf( F, "   <right_color G=\"%d %d\" R=\"%d %d\" globalID=\"%d\" B=\"%d %d\" />\n", (int)(256.0f*m_curve[i].cr[i1].col.y), (int)(256.0f*m_curve[i].cr[i1].col2.y), (int)(256.0f*m_curve[i].cr[i1].col.z), (int)(256.0f*m_curve[i].cr[i1].col2.z), m_curve[i].cr[i1].off, (int)(256.0f*m_curve[i].cr[i1].col.x), (int)(256.0f*m_curve[i].cr[i1].col2.x));
		}
		fprintf( F, "  </right_colors_set>\n");
				
		fprintf( F, "  <left_blur_points_set>\n");
		for (int i1=0; i1<m_curve[i].bLNum; i1++)
		{
			fprintf( F, "   <left_blur value=\"%d\" globalID=\"%d\" />\n", (int)(256.0f*m_curve[i].bL[i1].blurr), m_curve[i].bL[i1].off);
		}
		fprintf( F, "  </left_blur_points_set>\n");

		fprintf( F, "  <right_blur_points_set>\n");
		for (int i1=0; i1<m_curve[i].bRNum; i1++)
		{
			fprintf( F, "   <right_blur value=\"%d\" globalID=\"%d\" />\n", (int)(256.0f*m_curve[i].bR[i1].blurr), m_curve[i].bR[i1].off);
		}
		fprintf( F, "  </right_blur_points_set>\n");

		fprintf( F, " </curve>\n");
	}
	fprintf( F, "</curve_set>\n");
	fclose( F );
}







void VSObject::LoadVectorFile(ID3D10Device *pd3dDevice, char *s)
{
	ReadVectorFile( &s[0] );
	ConstructCurves(pd3dDevice);
}



void VSObject::ReadVectorFile( char *s )
{
	char buff[256];
	WCHAR wcFileInfo[512];
	char *token;
	FILE *F = fopen( s, "rb");
	bool gdci = false;
	while (fgets(buff, 255, F))
	{
		if (stringStartsWith(buff, "<!DOCTYPE CurveSetXML"))
		{
			StringCchPrintf( wcFileInfo, 512, L"(INFO) : This seems to be a diffusion curves file.\n");
			OutputDebugString( wcFileInfo );
			break;
		}
		if (stringStartsWith(buff, "<!DOCTYPE GDCICurveSetXML"))
		{
			StringCchPrintf( wcFileInfo, 512, L"(INFO) : This seems to be a generalized diffusion curves file.\n");
			OutputDebugString( wcFileInfo );
			gdci = true;
			break;
		}
	}
	bool invCrds = gdci;
	fgets(buff, 255, F);
	token = strtok(buff, " \"\t");
	while (!stringStartsWith(token, "image_width="))
		token = strtok(NULL, " \"\t");
	token = strtok(NULL, " \"\t");
	m_fWidth = atof(token);
	while (!stringStartsWith(token, "image_height="))
		token = strtok(NULL, " \"\t");
	token = strtok(NULL, " \"\t");
	m_fHeight = atof(token);
	while (!stringStartsWith(token, "nb_curves="))
		token = strtok(NULL, " \"\t");
	token = strtok(NULL, " \"\t");
	m_cNum = atof(token);
	StringCchPrintf( wcFileInfo, 512, L"(INFO) : %d curves found in file.\n", m_cNum);
	OutputDebugString( wcFileInfo );
	m_curve = new CURVE[m_cNum];
	m_cSegNum = 0;
	D3DXVECTOR2 maxBound = D3DXVECTOR2(-1000000,-1000000);
	D3DXVECTOR2 minBound = D3DXVECTOR2(1000000,1000000);
	float maxBlur = 0.0f;
	m_cPointNum = 0;
	m_bPointNum = 0;
	for (int i1=0; i1<m_cNum; i1++)
	{
		while (!stringStartsWith(buff, " <curve nb_control_points"))
			fgets(buff, 255, F);
		token = strtok(buff, " \"\t");
		while (!stringStartsWith(token, "nb_control_points="))
			token = strtok(NULL, " \"\t");
		token = strtok(NULL, " \"\t");
		m_curve[i1].pNum = atof(token);
		m_cSegNum += (m_curve[i1].pNum-1)/3;

		while (!stringStartsWith(token, "nb_left_colors="))
			token = strtok(NULL, " \"\t");
		token = strtok(NULL, " \"\t");
		m_curve[i1].clNum = atof(token);
		m_cPointNum += m_curve[i1].clNum;

		while (!stringStartsWith(token, "nb_right_colors="))
			token = strtok(NULL, " \"\t");
		token = strtok(NULL, " \"\t");
		m_curve[i1].crNum = atof(token);
		m_cPointNum += m_curve[i1].crNum;

		if (!gdci)
		{
			while (!stringStartsWith(token, "nb_blur_points="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].bLNum = atof(token);
			m_bPointNum += m_curve[i1].bLNum;
			m_curve[i1].bRNum = m_curve[i1].bLNum;
			m_bPointNum += m_curve[i1].bRNum;
		}
		else
		{
			while (!stringStartsWith(token, "nb_left_blur_points="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].bLNum = atof(token);
			m_bPointNum += m_curve[i1].bLNum;
			while (!stringStartsWith(token, "nb_right_blur_points="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].bRNum = atof(token);
			m_bPointNum += m_curve[i1].bRNum;
		}

		// read in individual curve data
		m_curve[i1].p = new D3DXVECTOR2[m_curve[i1].pNum];
		for (int i2=0; i2<m_curve[i1].pNum; i2++)
		{
			while (!stringStartsWith(buff, "   <control_point "))
				fgets(buff, 255, F);
			token = strtok(buff, " \"\t");
			while (!stringStartsWith(token, "x="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].p[i2].x = atof(token);
			while (!stringStartsWith(token, "y="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].p[i2].y = atof(token);
			if (invCrds == false)
			{
				float dummy = m_curve[i1].p[i2].x;
				m_curve[i1].p[i2].x = m_curve[i1].p[i2].y;
				m_curve[i1].p[i2].y = dummy;
			}
			fgets(buff, 255, F);
			//extend the bounds if necessary
			if (m_curve[i1].p[i2].y < minBound.y)
				minBound.y = m_curve[i1].p[i2].y;
			if (m_curve[i1].p[i2].y > maxBound.y)
				maxBound.y = m_curve[i1].p[i2].y;
			if (m_curve[i1].p[i2].x < minBound.x)
				minBound.x = m_curve[i1].p[i2].x;
			if (m_curve[i1].p[i2].x > maxBound.x)
				maxBound.x = m_curve[i1].p[i2].x;
		}
		m_curve[i1].cl = new COLORPOINT[m_curve[i1].clNum];
		for (int i2=0; i2<m_curve[i1].clNum; i2++)
		{
			while (!stringStartsWith(buff, "   <left_color "))
				fgets(buff, 255, F);
			token = strtok(buff, " \"\t");
			while (!stringStartsWith(token, "G="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cl[i2].col.y = atof(token)/256.0;
			if (!gdci)
			{
				m_curve[i1].cl[i2].col2.y = m_curve[i1].cl[i2].col.y;
			}
			else
			{
				token = strtok(NULL, " \"\t");
				m_curve[i1].cl[i2].col2.y = atof(token)/256.0;
			}
			while (!stringStartsWith(token, "R="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cl[i2].col.z  = atof(token)/256.0;
			if (!gdci)
			{
				m_curve[i1].cl[i2].col2.z = m_curve[i1].cl[i2].col.z;
			}
			else
			{
				token = strtok(NULL, " \"\t");
				m_curve[i1].cl[i2].col2.z = atof(token)/256.0;
			}
			while (!stringStartsWith(token, "globalID="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cl[i2].off = atof(token);

			while (!stringStartsWith(token, "B="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cl[i2].col.x  = atof(token)/256.0;
			if (!gdci)
			{
				m_curve[i1].cl[i2].col2.x = m_curve[i1].cl[i2].col.x;
			}
			else
			{
				token = strtok(NULL, " \"\t");
				m_curve[i1].cl[i2].col2.x = atof(token)/256.0;
			}
			fgets(buff, 255, F);
		}
		// sort the points according to curve parameter offset
		bool switched = true;
		while (switched)
		{
			switched = false;
			for (int i2=0; i2<m_curve[i1].clNum-1; i2++)
			{
				if (m_curve[i1].cl[i2].off > m_curve[i1].cl[i2+1].off)
				{
					D3DXVECTOR3 dCol = m_curve[i1].cl[i2+1].col;
					D3DXVECTOR3 dCol2 = m_curve[i1].cl[i2+1].col2;
					int dOff = m_curve[i1].cl[i2+1].off;
					m_curve[i1].cl[i2+1].col = m_curve[i1].cl[i2].col;
					m_curve[i1].cl[i2+1].col2 = m_curve[i1].cl[i2].col2;
					m_curve[i1].cl[i2+1].off = m_curve[i1].cl[i2].off;
					m_curve[i1].cl[i2].col = dCol;
					m_curve[i1].cl[i2].col2 = dCol2;
					m_curve[i1].cl[i2].off = dOff;
					switched = true;
				}
				if (m_curve[i1].cl[i2].off == m_curve[i1].cl[i2+1].off)
				{
					m_curve[i1].cl[i2].col = m_curve[i1].cl[m_curve[i1].clNum-1].col;
					m_curve[i1].cl[i2].col2 = m_curve[i1].cl[m_curve[i1].clNum-1].col2;
					m_curve[i1].cl[i2].off = m_curve[i1].cl[m_curve[i1].clNum-1].off;
					m_curve[i1].clNum--;
					m_cPointNum--;
					switched = true;
				}
			}
		}	
		m_curve[i1].cr = new COLORPOINT[m_curve[i1].crNum];
		for (int i2=0; i2<m_curve[i1].crNum; i2++)
		{
			while (!stringStartsWith(buff, "   <right_color "))
				fgets(buff, 255, F);
			token = strtok(buff, " \"\t");
			while (!stringStartsWith(token, "G="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cr[i2].col.y = atof(token)/256.0;
			if (!gdci)
			{
				m_curve[i1].cr[i2].col2.y = m_curve[i1].cr[i2].col.y;
			}
			else
			{
				token = strtok(NULL, " \"\t");
				m_curve[i1].cr[i2].col2.y = atof(token)/256.0;
			}
			while (!stringStartsWith(token, "R="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cr[i2].col.z  = atof(token)/256.0;
			if (!gdci)
			{
				m_curve[i1].cr[i2].col2.z = m_curve[i1].cr[i2].col.z;
			}
			else
			{
				token = strtok(NULL, " \"\t");
				m_curve[i1].cr[i2].col2.z = atof(token)/256.0;
			}
			while (!stringStartsWith(token, "globalID="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cr[i2].off = atof(token);

			while (!stringStartsWith(token, "B="))
				token = strtok(NULL, " \"\t");
			token = strtok(NULL, " \"\t");
			m_curve[i1].cr[i2].col.x  = atof(token)/256.0;
			if (!gdci)
			{
				m_curve[i1].cr[i2].col2.x = m_curve[i1].cr[i2].col.x;
			}
			else
			{
				token = strtok(NULL, " \"\t");
				m_curve[i1].cr[i2].col2.x = atof(token)/256.0;
			}
			fgets(buff, 255, F);
		}
		// sort the points according to curve paramter offset
		switched = true;
		while (switched)
		{
			switched = false;
			for (int i2=0; i2<m_curve[i1].crNum-1; i2++)
			{
				if (m_curve[i1].cr[i2].off > m_curve[i1].cr[i2+1].off)
				{
					D3DXVECTOR3 dCol = m_curve[i1].cr[i2+1].col;
					D3DXVECTOR3 dCol2 = m_curve[i1].cr[i2+1].col2;
					int dOff = m_curve[i1].cr[i2+1].off;
					m_curve[i1].cr[i2+1].col = m_curve[i1].cr[i2].col;
					m_curve[i1].cr[i2+1].col2 = m_curve[i1].cr[i2].col2;
					m_curve[i1].cr[i2+1].off = m_curve[i1].cr[i2].off;
					m_curve[i1].cr[i2].col = dCol;
					m_curve[i1].cr[i2].col2 = dCol2;
					m_curve[i1].cr[i2].off = dOff;
					switched = true;
				}
				if (m_curve[i1].cr[i2].off == m_curve[i1].cr[i2+1].off)
				{
					m_curve[i1].cr[i2].col = m_curve[i1].cr[m_curve[i1].crNum-1].col;
					m_curve[i1].cr[i2].col2 = m_curve[i1].cr[m_curve[i1].crNum-1].col2;
					m_curve[i1].cr[i2].off = m_curve[i1].cr[m_curve[i1].crNum-1].off;
					m_curve[i1].crNum--;
					m_cPointNum--;
					switched = true;
				}
			}
		}	

		m_curve[i1].bL = new BLURRPOINT[m_curve[i1].bLNum];
		m_curve[i1].bR = new BLURRPOINT[m_curve[i1].bRNum];
		if (!gdci)
		{
			for (int i2=0; i2<m_curve[i1].bLNum; i2++)
			{
				while (!stringStartsWith(buff, "   <best_scale"))
					fgets(buff, 255, F);
				token = strtok(buff, " \"\t");
				while (!stringStartsWith(token, "value="))
					token = strtok(NULL, " \"\t");
				token = strtok(NULL, " \"\t");
				m_curve[i1].bL[i2].blurr = atof(token);
				m_curve[i1].bR[i2].blurr = m_curve[i1].bL[i2].blurr;
				maxBlur = max(maxBlur,m_curve[i1].bL[i2].blurr);
				while (!stringStartsWith(token, "globalID="))
					token = strtok(NULL, " \"\t");
				token = strtok(NULL, " \"\t");
				m_curve[i1].bL[i2].off = atof(token);
				m_curve[i1].bR[i2].off = m_curve[i1].bL[i2].off;
				fgets(buff, 255, F);
			}
		}
		else
		{
			for (int i2=0; i2<m_curve[i1].bLNum; i2++)
			{
				while (!stringStartsWith(buff, "   <left_blur"))
					fgets(buff, 255, F);
				token = strtok(buff, " \"\t");
				while (!stringStartsWith(token, "value="))
					token = strtok(NULL, " \"\t");
				token = strtok(NULL, " \"\t");
				m_curve[i1].bL[i2].blurr = atof(token)/256.0;
	
				while (!stringStartsWith(token, "globalID="))
					token = strtok(NULL, " \"\t");
				token = strtok(NULL, " \"\t");
				m_curve[i1].bL[i2].off = atof(token);
				fgets(buff, 255, F);
			}
			for (int i2=0; i2<m_curve[i1].bRNum; i2++)
			{
				while (!stringStartsWith(buff, "   <right_blur"))
					fgets(buff, 255, F);
				token = strtok(buff, " \"\t");
				while (!stringStartsWith(token, "value="))
					token = strtok(NULL, " \"\t");
				token = strtok(NULL, " \"\t");
				m_curve[i1].bR[i2].blurr = atof(token)/256.0;
				
				while (!stringStartsWith(token, "globalID="))
					token = strtok(NULL, " \"\t");
				token = strtok(NULL, " \"\t");
				m_curve[i1].bR[i2].off = atof(token);
				fgets(buff, 255, F);
			}
		}
		// sort the points according to curve parameter offset
		switched = true;
		while (switched)
		{
			switched = false;
			for (int i2=0; i2<m_curve[i1].bLNum-1; i2++)
			{
				if (m_curve[i1].bL[i2].off > m_curve[i1].bL[i2+1].off)
				{
					float dBlurr = m_curve[i1].bL[i2+1].blurr;
					int dOff = m_curve[i1].bL[i2+1].off;
					m_curve[i1].bL[i2+1].blurr = m_curve[i1].bL[i2].blurr;
					m_curve[i1].bL[i2+1].off = m_curve[i1].bL[i2].off;
					m_curve[i1].bL[i2].blurr = dBlurr;
					m_curve[i1].bL[i2].off = dOff;
					switched = true;
				}
				if (m_curve[i1].bL[i2].off == m_curve[i1].bL[i2+1].off)
				{
					m_curve[i1].bL[i2].blurr = m_curve[i1].bL[m_curve[i1].bLNum-1].blurr;
					m_curve[i1].bL[i2].off = m_curve[i1].bL[m_curve[i1].bLNum-1].off;
					m_curve[i1].bLNum--;
					m_bPointNum--;
					switched = true;
				}
			}
		}	
		switched = true;
		while (switched)
		{
			switched = false;
			for (int i2=0; i2<m_curve[i1].bRNum-1; i2++)
			{
				if (m_curve[i1].bR[i2].off > m_curve[i1].bR[i2+1].off)
				{
					float dBlurr = m_curve[i1].bR[i2+1].blurr;
					int dOff = m_curve[i1].bR[i2+1].off;
					m_curve[i1].bR[i2+1].blurr = m_curve[i1].bR[i2].blurr;
					m_curve[i1].bR[i2+1].off = m_curve[i1].bR[i2].off;
					m_curve[i1].bR[i2].blurr = dBlurr;
					m_curve[i1].bR[i2].off = dOff;
					switched = true;
				}
				if (m_curve[i1].bR[i2].off == m_curve[i1].bR[i2+1].off)
				{
					m_curve[i1].bR[i2].blurr = m_curve[i1].bR[m_curve[i1].bRNum-1].blurr;
					m_curve[i1].bR[i2].off = m_curve[i1].bR[m_curve[i1].bRNum-1].off;
					m_curve[i1].bRNum--;
					m_bPointNum--;
					switched = true;
				}
			}
		}	
	}
	fclose(F);

	// scale the blur values between 0..1
	if ((!gdci) && (maxBlur > 0))
		for (int i1=0; i1<m_cNum; i1++)
		{
			for (int i2=0; i2<m_curve[i1].bLNum; i2++)
				m_curve[i1].bL[i2].blurr /= maxBlur;  //255.0
			for (int i2=0; i2<m_curve[i1].bRNum; i2++)
				m_curve[i1].bR[i2].blurr /= maxBlur;  //255.0
		}
	//scale the whole image to the input image
	for (int i1=0; i1<m_cNum; i1++)
		for (int i2=0; i2<m_curve[i1].pNum; i2++)
		{
			m_curve[i1].p[i2].x = 2.0f*(m_curve[i1].p[i2].x-m_fWidth/2.0)/m_fWidth;	
			m_curve[i1].p[i2].y = 2.0f*(m_curve[i1].p[i2].y-m_fHeight/2.0)/m_fHeight;
		}
	StringCchPrintf( wcFileInfo, 512, L"(INFO) : file contains %d color control points, %d blur control points and %d curve spline segments.\n", m_cPointNum, m_bPointNum, m_cSegNum);
	OutputDebugString( wcFileInfo );
	m_cPID = (D3DXVECTOR3**)(malloc(sizeof(D3DXVECTOR3*)*4*m_cPointNum));
	m_cPID2 = (D3DXVECTOR3**)(malloc(sizeof(D3DXVECTOR3*)*4*m_cPointNum));
	m_cPBID = (float**)(malloc(sizeof(float*)*4*m_bPointNum));
	m_cSplineID = (D3DXVECTOR2**)(malloc(sizeof(D3DXVECTOR2*)*4*m_cSegNum));
}




// convert the vectors into triangle strips and draw these
void VSObject::ConstructCurves(ID3D10Device *pd3dDevice)
{
	WCHAR wcFileInfo[512];
	D3DXVECTOR2	pLoop;
	int	iLoopStart;
	HRESULT	hr;
	if (pd!=NULL)
		delete[](pd);
	pd = new CURVE_Vertex[m_cSegNum*(int)(subSegNum)*10*2];
	int	*used = new int[m_cNum];
	ZeroMemory(used,m_cNum*sizeof(int));
	int lIDSum = 0;
	int rIDSum = 0;
	for (int i1=0; i1<m_cNum; i1++)
		rIDSum += m_curve[i1].clNum;
	int	lBIDSum = 0;
	int	rBIDSum = 0;
	for (int i1=0; i1<m_cNum; i1++)
		rBIDSum += m_curve[i1].bLNum;
	int cPos = 0;
	int cSplineID = 0;
	for (int iX=0; iX<m_cNum; iX++)
	{
		int i1=(int)(rand())*(m_cNum-1)/RAND_MAX;
		while (used[i1] > 0)
		{
			i1++;
			if (i1 == m_cNum)
				i1 = 0;
		}
		used[i1] = 1;
		int cSeg = 0;
		int lID = 0;
		while (m_curve[i1].cl[lID+1].off <= 0)
			lID++;
		int lS = m_curve[i1].cl[lID].off;
		int lN = m_curve[i1].cl[lID+1].off;
		int rID = 0;
		while (m_curve[i1].cr[rID+1].off <= 0)
			rID++;
		int rS = m_curve[i1].cr[rID].off;
		int rN = m_curve[i1].cr[rID+1].off;
		int lBID = 0;
		while (m_curve[i1].bL[lBID+1].off <= 0)
			lBID++;
		int lBS = m_curve[i1].bL[lBID].off;
		int lBN = m_curve[i1].bL[lBID+1].off;
		int rBID = 0;
		while (m_curve[i1].bR[rBID+1].off <= 0)
			rBID++;
		int rBS = m_curve[i1].bR[rBID].off;
		int rBN = m_curve[i1].bR[rBID+1].off;
		for (int i2=0; i2<m_curve[i1].pNum-1; i2+=3)
		{
			m_cSplineID[cSplineID*4+0] = &m_curve[i1].p[i2];
			m_cSplineID[cSplineID*4+1] = &m_curve[i1].p[i2+1];
			m_cSplineID[cSplineID*4+2] = &m_curve[i1].p[i2+2];
			m_cSplineID[cSplineID*4+3] = &m_curve[i1].p[i2+3];
			for (float t=0; t<1.0f; t+=0.1f)
			{
				// globally register the colors				
				m_cPID[lIDSum+lID] = &m_curve[i1].cl[lID].col;
				m_cPID2[lIDSum+lID] = &m_curve[i1].cl[lID].col2;
				m_cPID[lIDSum+lID+1] = &m_curve[i1].cl[lID+1].col;
				m_cPID2[lIDSum+lID+1] = &m_curve[i1].cl[lID+1].col2;
				m_cPID[rIDSum+rID] = &m_curve[i1].cr[rID].col;
				m_cPID2[rIDSum+rID] = &m_curve[i1].cr[rID].col2;
				m_cPID[rIDSum+rID+1] = &m_curve[i1].cr[rID+1].col;
				m_cPID2[rIDSum+rID+1] = &m_curve[i1].cr[rID+1].col2;
				m_cPBID[lBIDSum+lBID] = &m_curve[i1].bL[lBID].blurr;
				m_cPBID[lBIDSum+lBID+1] = &m_curve[i1].bL[lBID+1].blurr;
				m_cPBID[rBIDSum+rBID] = &m_curve[i1].bR[rBID].blurr;
				m_cPBID[rBIDSum+rBID+1] = &m_curve[i1].bR[rBID+1].blurr;
				float t1 = t;
				float t2 = t+0.1f;
				float fL = (float)(cSeg-lS)/(float)(lN-lS);
				float fLN = (float)(cSeg+1-lS)/(float)(lN-lS);
				D3DXVECTOR4 cL1 = D3DXVECTOR4((1.0f-fL)*m_curve[i1].cl[lID].col + fL*m_curve[i1].cl[lID+1].col, 1.0f);
				D3DXVECTOR4 cNL1 = D3DXVECTOR4((1.0f-fLN)*m_curve[i1].cl[lID].col + fLN*m_curve[i1].cl[lID+1].col, 1.0f);
				D3DXVECTOR4 cL2 = D3DXVECTOR4((1.0f-fL)*m_curve[i1].cl[lID].col2 + fL*m_curve[i1].cl[lID+1].col2, 1.0f);
				D3DXVECTOR4 cNL2 = D3DXVECTOR4((1.0f-fLN)*m_curve[i1].cl[lID].col2 + fLN*m_curve[i1].cl[lID+1].col2, 1.0f);
				float fR = (float)(cSeg-rS)/(float)(rN-rS);
				float fRN = (float)(cSeg+1-rS)/(float)(rN-rS);
				D3DXVECTOR4 cR1 = D3DXVECTOR4((1.0f-fR)*m_curve[i1].cr[rID].col + fR*m_curve[i1].cr[rID+1].col, 1.0f);
				D3DXVECTOR4 cNR1 = D3DXVECTOR4((1.0f-fRN)*m_curve[i1].cr[rID].col + fRN*m_curve[i1].cr[rID+1].col, 1.0f);
				D3DXVECTOR4 cR2 = D3DXVECTOR4((1.0f-fR)*m_curve[i1].cr[rID].col2 + fR*m_curve[i1].cr[rID+1].col2, 1.0f);
				D3DXVECTOR4 cNR2 = D3DXVECTOR4((1.0f-fRN)*m_curve[i1].cr[rID].col2 + fRN*m_curve[i1].cr[rID+1].col2, 1.0f);
				float fBL = (float)(cSeg-lBS)/(float)(lBN-lBS);
				float fBLN = (float)(cSeg+1-lBS)/(float)(lBN-lBS);
				float bL2 = (1.0f-fBL)*m_curve[i1].bL[lBID].blurr + fBL*m_curve[i1].bL[lBID+1].blurr;
				float bLN2 = (1.0f-fBLN)*m_curve[i1].bL[lBID].blurr + fBLN*m_curve[i1].bL[lBID+1].blurr;
				float fBR = (float)(cSeg-rBS)/(float)(rBN-rBS);
				float fBRN = (float)(cSeg+1-rBS)/(float)(rBN-rBS);
				float bR2 = (1.0f-fBR)*m_curve[i1].bR[rBID].blurr + fBR*m_curve[i1].bR[rBID+1].blurr;
				float bRN2 = (1.0f-fBRN)*m_curve[i1].bR[rBID].blurr + fBRN*m_curve[i1].bR[rBID+1].blurr;
				for (float sI=0.0f; sI<1.0f; sI+=1.0f/subSegNum)
				{
					float sN = sI+1.0f/subSegNum;
					float s1  = (1.0f-sI)*t1 + sI*t2;
					float s2 =  (1.0f-sN)*t1 + sN*t2;
					D3DXVECTOR2 p0 = (1.0f-s1)*(1.0f-s1)*(1.0f-s1)*m_curve[i1].p[i2] + 3*s1*(1.0f-s1)*(1.0f-s1)*m_curve[i1].p[i2+1] + 3*s1*s1*(1.0f-s1)*m_curve[i1].p[i2+2] + s1*s1*s1*m_curve[i1].p[i2+3];
					D3DXVECTOR2 p1 = (1.0f-s2)*(1.0f-s2)*(1.0f-s2)*m_curve[i1].p[i2] + 3*s2*(1.0f-s2)*(1.0f-s2)*m_curve[i1].p[i2+1] + 3*s2*s2*(1.0f-s2)*m_curve[i1].p[i2+2] + s2*s2*s2*m_curve[i1].p[i2+3];
					pd[cPos+0].col1[0] = (1.0f-sI)*cL1 + sI*cNL1;
					pd[cPos+1].col1[0] = (1.0f-sN)*cL1 + sN*cNL1;
					pd[cPos+0].col1[1] = (1.0f-sI)*cR1 + sI*cNR1;
					pd[cPos+1].col1[1] = (1.0f-sN)*cR1 + sN*cNR1;
					pd[cPos+0].col2[0] = (1.0f-sI)*cL2 + sI*cNL2;
					pd[cPos+1].col2[0] = (1.0f-sN)*cL2 + sN*cNL2;
					pd[cPos+0].col2[1] = (1.0f-sI)*cR2 + sI*cNR2;
					pd[cPos+1].col2[1] = (1.0f-sN)*cR2 + sN*cNR2;
					pd[cPos+0].col1[0].w = (1.0f-sI)*bL2 + sI*bLN2;
					pd[cPos+1].col1[0].w = (1.0f-sN)*bL2 + sN*bLN2;
					pd[cPos+0].col1[1].w = (1.0f-sI)*bR2 + sI*bRN2;
					pd[cPos+1].col1[1].w = (1.0f-sN)*bR2 + sN*bRN2;
					pd[cPos+0].col2[0].w = 0;
					pd[cPos+1].col2[0].w = 0;
					pd[cPos+0].col2[1].w = 0;
					pd[cPos+1].col2[1].w = 0;
					pd[cPos+0].pos = D3DXVECTOR2(p0.x,-p0.y);
					pd[cPos+1].pos = D3DXVECTOR2(p1.x,-p1.y);
					pd[cPos+0].IDw = D3DXVECTOR4(lIDSum+lID,  (1.0f-sI)*(1.0-fL) + sI*(1.0-fLN),  rIDSum+rID,  (1.0f-sI)*(1.0-fR) + sI*(1.0-fRN));
					pd[cPos+1].IDw = D3DXVECTOR4(lIDSum+lID,  (1.0f-sN)*(1.0-fL) + sN*(1.0-fLN),  rIDSum+rID,  (1.0f-sN)*(1.0-fR) + sN*(1.0-fRN));
					pd[cPos+0].IDwB = D3DXVECTOR4(lBIDSum+lBID,  (1.0f-sI)*(1.0-fBL) + sI*(1.0-fBLN),  rBIDSum+rBID,  (1.0f-sI)*(1.0-fBR) + sI*(1.0-fBRN));
					pd[cPos+1].IDwB = D3DXVECTOR4(lBIDSum+lBID,  (1.0f-sN)*(1.0-fBL) + sN*(1.0-fBLN),  rBIDSum+rBID,  (1.0f-sN)*(1.0-fBR) + sN*(1.0-fBRN));
					pd[cPos+0].IDwPos = D3DXVECTOR2((float)(cSplineID*4), s1);
					pd[cPos+1].IDwPos = D3DXVECTOR2((float)(cSplineID*4), s2);
					if ((i2 == 0) && (t == 0.0f) && (sI < 0.5f/subSegNum))
						pd[cPos+0].nb = D3DXVECTOR2(10000.0f, 10000.0f);
					else
					{
						pd[cPos+0].nb = pd[cPos-2].pos;
						pd[cPos-1].nb = pd[cPos+1].pos;
					}
					pd[cPos+1].nb = D3DXVECTOR2(10000.0f, 10000.0f);
					cPos += 2;
				}
				cSeg++;
				while ((cSeg >= lN) && (lID < m_curve[i1].clNum-2))
				{
					lID++;
					lS = m_curve[i1].cl[lID].off;
					lN = m_curve[i1].cl[lID+1].off;
				}
				while ((cSeg >= rN) && (rID < m_curve[i1].crNum-2))
				{
					rID++;
					rS = m_curve[i1].cr[rID].off;
					rN = m_curve[i1].cr[rID+1].off;
				}
				while ((cSeg >= lBN)  && (lBID < m_curve[i1].bLNum-2))
				{
					lBID++;
					lBS = m_curve[i1].bL[lBID].off;
					lBN = m_curve[i1].bL[lBID+1].off;
				}
				while ((cSeg >= rBN)  && (rBID < m_curve[i1].bRNum-2))
				{
					rBID++;
					rBS = m_curve[i1].bR[rBID].off;
					rBN = m_curve[i1].bR[rBID+1].off;
				}
			}
			cSplineID++;
		}
		lIDSum += m_curve[i1].clNum;
		rIDSum += m_curve[i1].crNum;
		lBIDSum += m_curve[i1].bLNum;
		rBIDSum += m_curve[i1].bRNum;
	}
	if (m_pMeshCurves)
	{
		m_pMeshCurves->Discard( D3DX10_MESH_DISCARD_ATTRIBUTE_BUFFER );
		m_pMeshCurves->Discard( D3DX10_MESH_DISCARD_ATTRIBUTE_TABLE );
		m_pMeshCurves->Discard( D3DX10_MESH_DISCARD_POINTREPS );
		m_pMeshCurves->Discard( D3DX10_MESH_DISCARD_ADJACENCY );
		m_pMeshCurves->Discard( D3DX10_MESH_DISCARD_DEVICE_BUFFERS );
	}
	m_pMeshCurves = NULL;
	V( D3DX10CreateMesh(pd3dDevice, VSObject::InputCurveElements, VSObject::InputCurveElementCount, "POSITION", m_cSegNum*(int)(subSegNum)*10*2, m_cSegNum*(int)(subSegNum)*10, 0, &m_pMeshCurves) );
	V( m_pMeshCurves->SetVertexData(0, pd) );
	V( m_pMeshCurves->CommitToDevice() );

	// delete all CPU data structures
	delete[]used;
	StringCchPrintf( wcFileInfo, 512, L"Finished.\n");
	OutputDebugString( wcFileInfo );
}






void VSObject::RenderDiffusion(ID3D10Device *pd3dDevice, int vID, int L_1_2)
{
	HRESULT hr;
	D3D10_TECHNIQUE_DESC techDesc;
    D3D10_PASS_DESC PassDesc;
	UINT stride, offset;
	
	//store the old render targets and viewports
    ID3D10RenderTargetView* old_pRTV = DXUTGetD3D10RenderTargetView();
    ID3D10DepthStencilView* old_pDSV = DXUTGetD3D10DepthStencilView();
	UINT NumViewports = 1;
	D3D10_VIEWPORT pViewports[100];
	pd3dDevice->RSGetViewports( &NumViewports, &pViewports[0]);
	pd3dDevice->RSSetViewports( 1, &m_vp );

	// set the shader variables, they are valid through the whole rendering pipeline
	V( g_pScale->SetFloat(m_scale) );
	V( g_pPan->SetFloatVector(m_pan) );
	V( m_pDiffX->SetFloat( m_sizeX ) );
	V( m_pDiffY->SetFloat( m_sizeY ) );
	V( m_pPolySize->SetFloat( m_polySize ) );
	V( m_pBlurScale->SetFloat( m_blurScale ) );
	V( m_pCmpColor->SetFloat( (float)(vID) ) );
	diffTex = 0;

	ID3D10InputLayout* pCurveVertexLayout;
	m_pDrawCurveDistancesTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
	if( FAILED( pd3dDevice->CreateInputLayout( InputCurveElements, InputCurveElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &pCurveVertexLayout ) ) )
		return;
	pd3dDevice->IASetInputLayout( pCurveVertexLayout );
  	ID3D10Buffer *pVertexBuffer;
	V( m_pMeshCurves->GetDeviceVertexBuffer(0, &pVertexBuffer) );
	stride = sizeof( CURVE_Vertex );
	offset = 0;
	pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
	pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_LINELIST );
	pd3dDevice->ClearDepthStencilView( m_pDepthStencilView, D3D10_CLEAR_DEPTH, 1.0f, 0 );
	V( m_pInTex->SetResource( NULL ) );
	m_pDiffuseDistancesTechnique->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->OMSetRenderTargets( 1, &m_diffuseTextureTV[1-diffTex], m_pDepthStencilView );
	m_pDrawCurveDistancesTechnique->GetDesc( &techDesc );
	for(UINT p=0; p<techDesc.Passes; p++)
	{
		m_pDrawCurveDistancesTechnique->GetPassByIndex( p )->Apply(0);
		pd3dDevice->Draw( m_pMeshCurves->GetVertexCount(), 0 );
	}
	diffTex = 1-diffTex;

	// setup the pipeline for the following image-space algorithms
	m_pDiffuseDistancesTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
	if( FAILED( pd3dDevice->CreateInputLayout( InputElements, InputElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pVertexLayout ) ) )
		return;
	pd3dDevice->IASetInputLayout( m_pVertexLayout );
	V( m_pMeshDiff->GetDeviceVertexBuffer(0, &pVertexBuffer) );
	stride = sizeof( VSO_Vertex );
	pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
	pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	// diffuse the distance to form the blend mask W_l
	for (int i=0; i<diffSteps; i++)
	{
		// Shrink always strategy (SA)
//		V( m_pPolySize->SetFloat( (float)(diffSteps-i)/(float)(diffSteps) ) );
		// Shrink half strategy (SH)
		V( m_pPolySize->SetFloat( 1.0 ) );
		if (i>diffSteps-diffSteps/2)
		{
			V( m_pPolySize->SetFloat( (float)(diffSteps-i)/(float)(diffSteps/2) ) );
		}
		V( m_pInTex->SetResource( NULL ) );
		m_pDiffuseDistancesTechnique->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->OMSetRenderTargets( 1, &m_diffuseTextureTV[1-diffTex], NULL );
		V( m_pInTex->SetResource( m_diffuseTextureRV[diffTex] ) );
		diffTex = 1-diffTex;
		m_pDiffuseDistancesTechnique->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
	}
	// transform function to smooth distance weight
	V( m_pDistDirTex->SetResource( NULL ) );
	V( m_pInTex->SetResource( NULL ) );
	m_pDistToYTechnique->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->OMSetRenderTargets( 1, &m_diffuseTextureTV[1-diffTex], NULL );
	V( m_pInTex->SetResource( m_diffuseTextureRV[diffTex] ) );
	m_pDistToYTechnique->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
	diffTex = 1-diffTex;
	pd3dDevice->CopyResource( m_distDirTexture, m_diffuseTexture[diffTex]);
	V( m_pDistDirTex->SetResource( m_distDirTextureRV ) );

	// now compute the individual color and blurr layers and diffuse them
	pd3dDevice->ClearRenderTargetView( m_resultTextureTV, D3DXVECTOR4(0,0,0,0)); 
	// this is for the first pass
	V( m_pInTex2->SetResource( m_distDirTextureRV ) );
	for (int i1=0; i1<2; i1++)
	{
		// this is only valid for curve rasterization for showing individual curve layers
		if (L_1_2 == 0) 
		{
			if (i1==0)
			{
				m_pShowLayers->SetFloat( 0.0f );
				m_pColLScale->SetFloat( m_1L);
				m_pColRScale->SetFloat( m_1R);
			}
			else
			{
				m_pShowLayers->SetFloat( 1.0f );
				m_pColLScale->SetFloat( m_2L);
				m_pColRScale->SetFloat( m_2R);
			}
		}
		else if (L_1_2 == 1) 
		{
			m_pShowLayers->SetFloat( 0.0f );
			m_pColLScale->SetFloat( m_1L);
			m_pColRScale->SetFloat( m_1R);
		}
		else if (L_1_2 == 2) 
		{
			m_pShowLayers->SetFloat( 1.0f );
			m_pColLScale->SetFloat( m_2L);
			m_pColRScale->SetFloat( m_2R);
		}
		pCurveVertexLayout;
		m_pDrawCurveColorsTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
		if( FAILED( pd3dDevice->CreateInputLayout( InputCurveElements, InputCurveElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &pCurveVertexLayout ) ) )
			return;
		pd3dDevice->IASetInputLayout( pCurveVertexLayout );
		V( m_pMeshCurves->GetDeviceVertexBuffer(0, &pVertexBuffer) );
		stride = sizeof( CURVE_Vertex );
		offset = 0;
		pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
		pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_LINELIST );
		V( m_pInTex->SetResource( NULL ) );
		V( m_pPolySize->SetFloat( m_polySize ) );
		m_pDrawCurveColorsTechnique->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->OMSetRenderTargets( 1, &m_diffuseTextureTV[1-diffTex], m_pDepthStencilView );
		m_pDrawCurveColorsTechnique->GetDesc( &techDesc );
		for(UINT p=0; p<techDesc.Passes; p++)
		{
			m_pDrawCurveColorsTechnique->GetPassByIndex( p )->Apply(0);
			pd3dDevice->Draw( m_pMeshCurves->GetVertexCount(), 0 );
		}
		diffTex = 1-diffTex;

		// proceed diffusion with the "right" curve layer parameter
		if (i1==0)
		{
			m_pShowLayers->SetFloat( 0.0f );
			m_pColLScale->SetFloat( m_1L);
			m_pColRScale->SetFloat( m_1R);
		}
		else
		{
			m_pShowLayers->SetFloat( 1.0f );
			m_pColLScale->SetFloat( m_2L);
			m_pColRScale->SetFloat( m_2R);
		}

		// setup the pipeline for the following image-space algorithms
		m_pDiffuseColorsTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
		if( FAILED( pd3dDevice->CreateInputLayout( InputElements, InputElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pVertexLayout ) ) )
			return;
		pd3dDevice->IASetInputLayout( m_pVertexLayout );
		V( m_pMeshDiff->GetDeviceVertexBuffer(0, &pVertexBuffer) );
		stride = sizeof( VSO_Vertex );
		offset = 0;
		pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
		pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
		// diffuse the colors and blur
		for (int i=0; i<diffSteps2; i++)
		{
			// SA strategy
//			V( m_pPolySize->SetFloat( (float)(diffSteps2-i)/(float)(diffSteps2) ) );
			// SH strategy
			V( m_pPolySize->SetFloat( 1.0 ) );
			if (i>diffSteps2-diffSteps2/2)
			{
				V( m_pPolySize->SetFloat( (float)(diffSteps2-i)/(float)(diffSteps2/2) ) );
			}
			V( m_pInTex->SetResource( NULL ) );
			m_pDiffuseColorsTechnique->GetPassByIndex( 0 )->Apply(0);
			pd3dDevice->OMSetRenderTargets( 1, &m_diffuseTextureTV[1-diffTex], NULL );
			V( m_pInTex->SetResource( m_diffuseTextureRV[diffTex] ) );
			m_pDiffuseColorsTechnique->GetPassByIndex( 0 )->Apply(0);
			pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
			diffTex = 1-diffTex;
		}	
		// add this layer to the output
		V( m_pInTex->SetResource( NULL ) );
		m_pAddToOutputTechnique->GetPassByIndex( 0 )->Apply(0);
		V( m_pInTex->SetResource( m_diffuseTextureRV[diffTex] ) );
		pd3dDevice->CopyResource( m_diffuseTexture[1-diffTex], m_resultTexture);
		V( m_pInTex2->SetResource( m_diffuseTextureRV[1-diffTex] ) );
		pd3dDevice->OMSetRenderTargets( 1, &m_resultTextureTV, NULL );
		m_pAddToOutputTechnique->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
		// this is for blurr map computation in the second path
		V( m_pInTex2->SetResource( m_resultTextureRV ) );
	}		
	V( m_pPolySize->SetFloat( m_polySize ) );

	// anti aliasing for non-blurred boundaries
	pd3dDevice->CopyResource( m_diffuseTexture[diffTex], m_resultTexture);
	m_pInTex->SetResource( m_diffuseTextureRV[diffTex] );
	pd3dDevice->OMSetRenderTargets( 1, &m_resultTextureTV, NULL );
	V( m_pDistDirTex->SetResource( m_distDirTextureRV ) );
	m_pAntiAliasTechnique->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );

	// setup the pipeline for blur distance computation
	m_pBlurrDistTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
	if( FAILED( pd3dDevice->CreateInputLayout( ScreenSpaceElements, ScreenSpaceElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pVertexLayout ) ) )
		return;
	pd3dDevice->IASetInputLayout( m_pVertexLayout );
	V( m_pDisplayMesh->GetDeviceVertexBuffer(0, &pVertexBuffer) );
	stride = sizeof( SIMPLE_Vertex );
	offset = 0;
	pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
	pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_POINTLIST );
	pd3dDevice->ClearRenderTargetView( m_otherTextureTV, D3DXVECTOR4(0,0,0,0));
	pd3dDevice->ClearDepthStencilView( m_pDepthStencilView, D3D10_CLEAR_DEPTH, 1.0f, 0 );
	V( m_pInTex->SetResource( m_resultTextureRV ) );
	m_pBlurrDistTechnique->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->OMSetRenderTargets( 1, &m_otherTextureTV, m_pDepthStencilView );
	m_pBlurrDistTechnique->GetDesc( &techDesc );
	for(UINT p=0; p<techDesc.Passes; p++)
	{
		m_pBlurrDistTechnique->GetPassByIndex( p )->Apply(0);
		pd3dDevice->Draw( m_pDisplayMesh->GetVertexCount(), 0 );
	}
	// setup the pipeline for the following image-space algorithms
	m_pBlurTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
	if( FAILED( pd3dDevice->CreateInputLayout( InputElements, InputElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pVertexLayout ) ) )
		return;
	pd3dDevice->IASetInputLayout( m_pVertexLayout );
	V( m_pMeshDiff->GetDeviceVertexBuffer(0, &pVertexBuffer) );
	stride = sizeof( VSO_Vertex );
	offset = 0;
	pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
	pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	V( m_pInTex2->SetResource( m_otherTextureRV ) );

	// finally, blurr the result image
	pd3dDevice->CopyResource( m_diffuseTexture[diffTex], m_resultTexture);
	// edge blur as biharmonic function
	for (int i=0; i<diffSteps3; i++)
	{
		// SA strategy
//		V( m_pPolySize->SetFloat( (float)(diffSteps3-i)/(float)(diffSteps3) ) );
		// SH strategy
		V( m_pPolySize->SetFloat( 1.0 ) );
		if (i>diffSteps3-diffSteps3/2)
			m_pPolySize->SetFloat( (float)(diffSteps3-i)/(float)(diffSteps3/2) );
		V( m_pInTex->SetResource( NULL ) );
		m_pBlurTechnique->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->OMSetRenderTargets( 1, &m_diffuseTextureTV[1-diffTex], NULL );
		V( m_pDistDirTex->SetResource( m_otherTextureRV ) );
		m_pInTex2->SetResource( m_resultTextureRV );
		m_pInTex->SetResource( m_diffuseTextureRV[diffTex] );
		diffTex = 1-diffTex;
		m_pBlurTechnique->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
	}
	
	// copy final texture to resultTexture
	pd3dDevice->CopyResource( m_resultTexture, m_diffuseTexture[diffTex] );

	// cleanup		
	V( m_pInTex->SetResource( NULL ) );
	V( m_pInTex2->SetResource( NULL ) );
	V( m_pDistDirTex->SetResource( NULL ) );
	m_pDiffuseColorsTechnique->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->OMSetRenderTargets( 1,  &old_pRTV,  old_pDSV );
	pd3dDevice->RSSetViewports( NumViewports, &pViewports[0]);
}





// this renders the final image to the screen
void VSObject::Render(ID3D10Device *pd3dDevice, bool showLayerWeight, bool showBlurMap, bool showBlurDist, bool showCurves, bool showControlPoints, int mode)
{
	HRESULT hr;
	// Create the input layout
    D3D10_PASS_DESC PassDesc;
    m_pDisplayImage->GetPassByIndex( 0 )->GetDesc( &PassDesc );
    if( FAILED( pd3dDevice->CreateInputLayout( InputElements, InputElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pVertexLayout ) ) )
        return;
    pd3dDevice->IASetInputLayout( m_pVertexLayout );
	V( m_pMeshDiff->GetDeviceVertexBuffer(0, &m_pVertexBuffer) );
    UINT stride = sizeof( VSO_Vertex );
    UINT offset = 0;
    pd3dDevice->IASetVertexBuffers( 0, 1, &m_pVertexBuffer, &stride, &offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	V( m_pDistDirTex->SetResource( m_distDirTextureRV ) );

	V( m_pInTex->SetResource( m_resultTextureRV ) );
	m_pShowLayers->SetFloat( 0.0f );
	if (showBlurMap)
	{
		m_pShowLayers->SetFloat( 1.0f );
	}
	if (showLayerWeight)
	{
		V( m_pInTex->SetResource( m_distDirTextureRV ) );
		m_pShowLayers->SetFloat( 2.0f );
	}
	if (showBlurDist)
	{
		V( m_pInTex->SetResource( m_otherTextureRV ) );
		m_pShowLayers->SetFloat( 3.0f );
	}
	m_pDisplayImage->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
	if (showCurves)
	{
		// set the shader variables, they are valid through the whole rendering pipeline
		V( g_pScale->SetFloat(m_scale) );
		V( g_pPan->SetFloatVector(m_pan) );
		V( m_pDiffX->SetFloat( m_sizeX ) );
		V( m_pDiffY->SetFloat( m_sizeY ) );
		V( m_pPolySize->SetFloat( m_polySize ) );
		ID3D10InputLayout* pCurveVertexLayout;
		m_pDrawCurveColorsTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
		if( FAILED( pd3dDevice->CreateInputLayout( InputCurveElements, InputCurveElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &pCurveVertexLayout ) ) )
			return;
		pd3dDevice->IASetInputLayout( pCurveVertexLayout );
  		ID3D10Buffer *pVertexBuffer;
		V( m_pMeshCurves->GetDeviceVertexBuffer(0, &pVertexBuffer) );
		stride = sizeof( CURVE_Vertex );
		offset = 0;
		pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
		pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_LINELIST );
		//construct the curve triangles in the geometry shader and render them directly
		m_pDrawCurvesOverlay->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->Draw( m_pMeshCurves->GetVertexCount(), 0 );
	}

	// show the control points
	if (showControlPoints)
	{
		float cpScale = 12.0f;
		// first grab all the colors from the curves
		int cpNum = 0;
		if ((mode == COLOR_MODE) || (mode == LIGHTNESS_MODE))
			for (int i1=0; i1<m_cNum; i1++)
			{
				for (int i2=0; i2<m_curve[i1].clNum; i2++)
				{
					int pID = 3*(m_curve[i1].cl[i2].off/10);
					float t = (m_curve[i1].cl[i2].off%10)/10.0f;
					if (pID >= m_curve[i1].pNum-1)
					{
						pID -= 3;
						t = 1.0f;
					}
					D3DXVECTOR2 p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					t += 0.1f;
					D3DXVECTOR2 p1 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					D3DXVECTOR2 offset;
					D3DXVec2Normalize( &offset, &(p1-p0));
					m_controlPointData[cpNum].pos = D3DXVECTOR2(p0.x,p0.y) + 1.0f/m_scale*13.0f/1000.0f*D3DXVECTOR2(offset.y,-offset.x); 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col = 0.5*(m_curve[i1].cl[i2].col + m_curve[i1].cl[i2].col2);
					cpNum++;
				}
				for (int i2=0; i2<m_curve[i1].crNum; i2++)
				{
					int pID = 3*(m_curve[i1].cr[i2].off/10);
					float t = (m_curve[i1].cr[i2].off%10)/10.0f;
					if (pID >= m_curve[i1].pNum-1)
					{
						pID -= 3;
						t = 1.0f;
					}
					D3DXVECTOR2 p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					t += 0.1f;
					D3DXVECTOR2 p1 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					D3DXVECTOR2 offset;
					D3DXVec2Normalize( &offset, &(p1-p0));
					m_controlPointData[cpNum].pos = D3DXVECTOR2(p0.x,p0.y) - 1.0f/m_scale*13.0f/1000.0f*D3DXVECTOR2(offset.y,-offset.x); 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col = 0.5*(m_curve[i1].cr[i2].col + m_curve[i1].cr[i2].col2);
					cpNum++;
				}
			}
		if (mode == BLUR_MODE)
			for (int i1=0; i1<m_cNum; i1++)
			{
				for (int i2=0; i2<m_curve[i1].bLNum; i2++)
				{
					int pID = 3*(m_curve[i1].bL[i2].off/10);
					float t = (m_curve[i1].bL[i2].off%10)/10.0f;
					if (pID >= m_curve[i1].pNum-1)
					{
						pID -= 3;
						t = 1.0f;
					}
					D3DXVECTOR2 p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					t += 0.1f;
					D3DXVECTOR2 p1 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					D3DXVECTOR2 offset;
					D3DXVec2Normalize( &offset, &(p1-p0));
					m_controlPointData[cpNum].pos = D3DXVECTOR2(p0.x,p0.y) + 1.0f/m_scale*13.0f/1000.0f*D3DXVECTOR2(offset.y,-offset.x); 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col.x = m_curve[i1].bL[i2].blurr;
					m_controlPointData[cpNum].col.y = m_controlPointData[cpNum].col.x;
					m_controlPointData[cpNum].col.z = m_controlPointData[cpNum].col.x;
					cpNum++;
				}
				for (int i2=0; i2<m_curve[i1].bRNum; i2++)
				{
					int pID = 3*(m_curve[i1].bR[i2].off/10);
					float t = (m_curve[i1].bR[i2].off%10)/10.0f;
					if (pID >= m_curve[i1].pNum-1)
					{
						pID -= 3;
						t = 1.0f;
					}
					D3DXVECTOR2 p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					t += 0.1f;
					D3DXVECTOR2 p1 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					D3DXVECTOR2 offset;
					D3DXVec2Normalize( &offset, &(p1-p0));
					m_controlPointData[cpNum].pos = D3DXVECTOR2(p0.x,p0.y) - 1.0f/m_scale*13.0f/1000.0f*D3DXVECTOR2(offset.y,-offset.x); 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col.x = m_curve[i1].bR[i2].blurr;
					m_controlPointData[cpNum].col.y = m_controlPointData[cpNum].col.x;
					m_controlPointData[cpNum].col.z = m_controlPointData[cpNum].col.x;
					cpNum++;
				}
			}
		if (mode == CURVE_MODE)
		{
			cpScale = 8.0f;
			for (int i1=0; i1<m_cNum; i1++)
				for (int i2=0; i2<m_curve[i1].pNum-1; i2+=3)
				{
					int pID = i2;
					float t = 0;
					D3DXVECTOR2 p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					m_controlPointData[cpNum].pos = p0; 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col = D3DXVECTOR3(0,0,0);
					cpNum++;
					t = 0.33f;
					p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					m_controlPointData[cpNum].pos = p0; 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col = D3DXVECTOR3(-10000,-10000,-10000);
					cpNum++;
					t = 0.66f;
					p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					m_controlPointData[cpNum].pos = p0; 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col = D3DXVECTOR3(-10000,-10000,-10000);
					cpNum++;
					t = 1.0f;
					p0 = (1.0f-t)*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID] + 3*t*(1.0f-t)*(1.0f-t)*m_curve[i1].p[pID+1] + 3*t*t*(1.0f-t)*m_curve[i1].p[pID+2] + t*t*t*m_curve[i1].p[pID+3];
					m_controlPointData[cpNum].pos = p0; 
					m_controlPointData[cpNum].pos.y = -m_controlPointData[cpNum].pos.y;
					m_controlPointData[cpNum].pos *= m_scale;
					m_controlPointData[cpNum].pos += m_pan;
					m_controlPointData[cpNum].col = D3DXVECTOR3(0,0,0);
					cpNum++;
				}
		}
		V( m_pMeshControlPoints->SetVertexData(0, m_controlPointData) );
		V( m_pMeshControlPoints->CommitToDevice() );

		D3D10_PASS_DESC PassDesc;
		ID3D10InputLayout* pControlPointVertexLayout;
		m_pDrawControlPointTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc );
		if( FAILED( pd3dDevice->CreateInputLayout( ControlPointElementLayout, InputControlPointElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &pControlPointVertexLayout ) ) )
			return;
		pd3dDevice->IASetInputLayout( pControlPointVertexLayout );
		ID3D10Buffer *pVertexBuffer;
		V( m_pMeshControlPoints->GetDeviceVertexBuffer(0, &pVertexBuffer) );
		UINT stride = sizeof( CONTROLPOINT_Vertex );
		UINT offset = 0;
		pd3dDevice->IASetVertexBuffers( 0, 1, &pVertexBuffer, &stride, &offset );
		pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_POINTLIST );
		V( g_pScale->SetFloat(cpScale) );
		m_pDrawControlPointTechnique->GetPassByIndex( 0 )->Apply(0);
		pd3dDevice->Draw( cpNum, 0 );
		V( g_pScale->SetFloat(m_scale) );
	}
	V( m_pDistDirTex->SetResource( NULL ) );
	V( m_pInTex->SetResource( NULL ) );
	m_pDisplayImage->GetPassByIndex( 0 )->Apply(0);
}



// ******************************************************************
// Display and editing functions (palette etc.)
// ******************************************************************


// this draws the mouse on top for video recording
void VSObject::DisplayMouse(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse)
{
	HRESULT hr;
	// Create the input layout
    D3D10_PASS_DESC PassDesc;
    m_pDisplayMouse->GetPassByIndex( 0 )->GetDesc( &PassDesc );
    if( FAILED( pd3dDevice->CreateInputLayout( InputElements, InputElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pVertexLayout ) ) )
        return;
    pd3dDevice->IASetInputLayout( m_pVertexLayout );
	if(m_pVertexBuffer == NULL)
		V( m_pMeshDiff->GetDeviceVertexBuffer(0, &m_pVertexBuffer) );
    UINT stride = sizeof( VSO_Vertex );
    UINT offset = 0;
    pd3dDevice->IASetVertexBuffers( 0, 1, &m_pVertexBuffer, &stride, &offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	V( g_pScale->SetFloat(1.0f) );
	V( g_pPan->SetFloatVector( D3DXVECTOR2(mouse.x,mouse.y)) );
	m_pDisplayMouse->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
}



// this renders the texture to the output
void VSObject::DisplayPalette(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse, int mode)
{
	// set viewport to right window half 
	UINT NumViewports = 1;
	D3D10_VIEWPORT pViewports[100];
	pd3dDevice->RSGetViewports( &NumViewports, &pViewports[0]);
	D3D10_VIEWPORT vp;
    vp.Width = 256;
    vp.Height = 256;
    vp.MinDepth = 0;
    vp.MaxDepth = 1.0f;
	UINT scrSizeX = DXUTGetDXGIBackBufferSurfaceDesc()->Width;
	UINT scrSizeY = DXUTGetDXGIBackBufferSurfaceDesc()->Height;
    vp.TopLeftX = scrSizeX-256;
    vp.TopLeftY = scrSizeY-256;
    pd3dDevice->RSSetViewports( 1, &vp );

	HRESULT hr;
	// Create the input layout
    D3D10_PASS_DESC PassDesc;
    m_pDisplayImage->GetPassByIndex( 0 )->GetDesc( &PassDesc );
    if( FAILED( pd3dDevice->CreateInputLayout( InputElements, InputElementCount, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pVertexLayout ) ) )
        return;
    pd3dDevice->IASetInputLayout( m_pVertexLayout );
	if(m_pVertexBuffer == NULL)
		V( m_pMeshDiff->GetDeviceVertexBuffer(0, &m_pVertexBuffer) );
    UINT stride = sizeof( VSO_Vertex );
    UINT offset = 0;
    pd3dDevice->IASetVertexBuffers( 0, 1, &m_pVertexBuffer, &stride, &offset );
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	V( g_pScale->SetFloat(1.0f) );
	V( g_pPan->SetFloatVector(D3DXVECTOR2(0,0)) );
	V( m_pCmpColor->SetFloat( 1 ) );
	if (mode == COLOR_MODE)
	{
		int x = (int)(m_pickedPalMouse.x + mouse.x*scrSizeX - m_pickedMouse.x);
		int y = (int)(m_pickedPalMouse.y + mouse.y*scrSizeY - m_pickedMouse.y);
		x = max(0,min(256.0f,x));
		y = max(0,min(256.0f,y));
		V( g_pPan->SetFloatVector( D3DXVECTOR2(x,y)/256.0f) );
		V( m_pCmpColor->SetFloat( 0 ) );
	}
	if (mode == LIGHTNESS_MODE)
	{
		float y = -(mouse.y*m_sizeY - m_pickedMouse.y)*(m_sizeY/256.0f)/(m_sizeY);
		float maxVal = m_pickedPalCol.x + y; 
		maxVal = 1.0f-max(0.0f,min(1.0f,maxVal));
		V( g_pPan->SetFloatVector( D3DXVECTOR2(0,maxVal)) );
		V( m_pCmpColor->SetFloat( 1 ) );
	}
	if (mode == BLUR_MODE)
	{
		float y = -(mouse.y*m_sizeY - m_pickedMouse.y)*(m_sizeY/256.0f)/(m_sizeY);
		float maxVal = m_pickedBlurVal + y; 
		maxVal = 1.0f-max(0.0f,min(1.0f,maxVal));
		V( g_pPan->SetFloatVector( D3DXVECTOR2(0,maxVal)) );
		V( m_pCmpColor->SetFloat( 1 ) );
	}
	m_pDisplayPalette->GetPassByIndex( 0 )->Apply(0);
	pd3dDevice->Draw( 3*m_pMeshDiff->GetFaceCount(), 0 );
	// set viewport back to normal
	pd3dDevice->RSSetViewports( NumViewports, &pViewports[0]);
}







void  VSObject::SetMousePicker(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse, int mode)
{
	HRESULT hr;
	UINT scrSizeX = DXUTGetDXGIBackBufferSurfaceDesc()->Width;
	UINT scrSizeY = DXUTGetDXGIBackBufferSurfaceDesc()->Height;
	m_pickedMouse = D3DXVECTOR2(mouse.x*scrSizeX, mouse.y*scrSizeY);  // we store this in SCREEN coordinates as later we refer to those
	int x = (int)(mouse.x*m_sizeX);
	int y = (int)(mouse.y*m_sizeY);

	// get the current mouse position in palette coordinates
	if (mode == COLOR_MODE)
	{
		pd3dDevice->CopyResource( m_fitCPUTexture, m_resultTexture);
		D3D10_MAPPED_TEXTURE2D pMappedTex;
		V( ((ID3D10Texture2D*)(m_fitCPUTexture))->Map( 0, D3D10_MAP_READ, 0, &pMappedTex) );
		float *pData = (float*)(pMappedTex.pData);
		D3DXVECTOR4 pickedCol;
		pickedCol.x = pData[y*(int)(scrSizeX)*4+x*4+0];  
		pickedCol.y = pData[y*(int)(scrSizeX)*4+x*4+1];  
		pickedCol.z = pData[y*(int)(scrSizeX)*4+x*4+2];  
		((ID3D10Texture2D*)(m_fitCPUTexture))->Unmap( 0 );
		float bestDist = 256*256*3;
		int bestX = 0;
		int bestY = 0;
		for (int y1=0; y1<256; y1++)
			for (int x1=0; x1<256; x1++)
			{
				float dist =  (m_pPaletteImage[y1*256*4+x1*4+0]-pickedCol.x)*(m_pPaletteImage[y1*256*4+x1*4+0]-pickedCol.x) 
							+ (m_pPaletteImage[y1*256*4+x1*4+1]-pickedCol.y)*(m_pPaletteImage[y1*256*4+x1*4+1]-pickedCol.y) 
							+ (m_pPaletteImage[y1*256*4+x1*4+2]-pickedCol.z)*(m_pPaletteImage[y1*256*4+x1*4+2]-pickedCol.z);
				if (dist <= bestDist)
				{
					bestX = x1;
					bestY = y1;
					bestDist = dist;
				}
			}
		m_pickedPalMouse = D3DXVECTOR2(bestX, bestY);
		m_pickedPalCol = D3DXVECTOR3(m_pPaletteImage[bestY*256*4+bestX*4+0], m_pPaletteImage[bestY*256*4+bestX*4+1], m_pPaletteImage[bestY*256*4+bestX*4+2]);
		m_pickedPalCol.x = max(0.001f, m_pickedPalCol.x);
		m_pickedPalCol.y = max(0.001f, m_pickedPalCol.y);
		m_pickedPalCol.z = max(0.001f, m_pickedPalCol.z);
	}
	if (mode == LIGHTNESS_MODE)
	{
		pd3dDevice->CopyResource( m_fitCPUTexture, m_resultTexture);
		D3D10_MAPPED_TEXTURE2D pMappedTex;
		V( ((ID3D10Texture2D*)(m_fitCPUTexture))->Map( 0, D3D10_MAP_READ, 0, &pMappedTex) );
		float *pData = (float*)(pMappedTex.pData);
		D3DXVECTOR4 pickedCol;
		pickedCol.x = pData[y*(int)(scrSizeX)*4+x*4+0];  
		pickedCol.y = pData[y*(int)(scrSizeX)*4+x*4+1];  
		pickedCol.z = pData[y*(int)(scrSizeX)*4+x*4+2];  
		((ID3D10Texture2D*)(m_fitCPUTexture))->Unmap( 0 );
		m_pickedPalCol.x = (pickedCol.x+pickedCol.y+pickedCol.z)*0.333333;
		m_pickedPalCol.y = m_pickedPalCol.x;
		m_pickedPalCol.z = m_pickedPalCol.x;
	}

	// get the picked depth and IDs
	pd3dDevice->CopyResource( m_fitCPUTexture, m_distDirTexture);
	D3D10_MAPPED_TEXTURE2D pMappedTex;
	V( ((ID3D10Texture2D*)(m_fitCPUTexture))->Map( 0, D3D10_MAP_READ, 0, &pMappedTex) );
	float *pData = (float*)(pMappedTex.pData);
	m_pickedDepth = pData[y*pMappedTex.RowPitch/4+x*4+3];  
	((ID3D10Texture2D*)(m_fitCPUTexture))->Unmap( 0 );
	float pX,pY;
	pX = 2.0*(float)(mouse.x)-1.0;
	pY = -(2.0*(float)(mouse.y)-1.0);
	pX -= m_pan.x;
	pX /= m_scale;
	pY -= m_pan.y;
	pY /= m_scale;
	GetNearestSegmentID( pX, pY, m_pickedID, m_pickedWeight, m_pickedIDPos, m_pickedWPos, m_pickedIDBlur, m_pickedWBlur);
	// store the influenced colors
	m_pickedCol1L1 = *m_cPID[m_pickedID];
	m_pickedCol2L1 = *m_cPID[m_pickedID+1];
	m_pickedCol1L2 = *m_cPID2[m_pickedID];
	m_pickedCol2L2 = *m_cPID2[m_pickedID+1];
	// store the influence blur
	m_pickedBlur1 = *m_cPBID[m_pickedIDBlur];
	m_pickedBlur2 = *m_cPBID[m_pickedIDBlur+1];
	m_pickedBlurVal = m_pickedBlur1*(1-m_pickedWBlur) + m_pickedBlur2*m_pickedWBlur;
	// store spline point position
	m_pickedPoint1 = *m_cSplineID[m_pickedIDPos+0];
	m_pickedPoint2 = *m_cSplineID[m_pickedIDPos+1];
	m_pickedPoint3 = *m_cSplineID[m_pickedIDPos+2];
	m_pickedPoint4 = *m_cSplineID[m_pickedIDPos+3];
	if (m_pickedWPos < 0.5*0.33)
		m_pickedIDPos += 0;
	else if (m_pickedWPos < 0.33+0.5*0.33)
		m_pickedIDPos += 1;
	else if (m_pickedWPos < 0.66+0.5*0.33)
		m_pickedIDPos += 2;
	else  
		m_pickedIDPos += 3;
	m_pickedPointPos = *m_cSplineID[m_pickedIDPos];
}






void  VSObject::SetPickedColors(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse, int mode)
{
	if ((mode == COLOR_MODE) || (mode == LIGHTNESS_MODE))
	{
		int x = max(0,min(255,(int)(m_pickedPalMouse.x + mouse.x*m_sizeX - m_pickedMouse.x)));
		int y = max(0,min(255,(int)(m_pickedPalMouse.y + mouse.y*m_sizeY - m_pickedMouse.y)));
		D3DXVECTOR3 colorChange;
		if (mode == COLOR_MODE)
			colorChange = D3DXVECTOR3(m_pPaletteImage[y*256*4+x*4+0]-m_pickedPalCol.x, m_pPaletteImage[y*256*4+x*4+1]-m_pickedPalCol.y, m_pPaletteImage[y*256*4+x*4+2]-m_pickedPalCol.z); 
		if (mode == LIGHTNESS_MODE)
		{
			float y = (mouse.y*m_sizeY - m_pickedMouse.y)*(m_sizeY/256.0f)/(m_sizeY);
			colorChange = -D3DXVECTOR3(y,y,y); 
		}
		// change both adjacent color control points
		float w = 1-m_pickedWeight;
		float wDepth = 0;		
		if (m_pickedDepth > 0.3)
			wDepth = 1.0;
		*m_cPID[m_pickedID] = m_pickedCol1L1 + (1-w)*(colorChange*(1-wDepth));
		*m_cPID[m_pickedID+1] = m_pickedCol2L1 + w*(colorChange*(1-wDepth));
		*m_cPID2[m_pickedID] = m_pickedCol1L2 + (1-w)*(colorChange*wDepth);
		*m_cPID2[m_pickedID+1] = m_pickedCol2L2 + w*(colorChange*wDepth);
		// assign new colors to all influenced PL curve points
		for (int i=0; i<m_cSegNum*(int)(subSegNum)*10*2; i++)
		{
			int idL = pd[i].IDw.x;
			float wL = pd[i].IDw.y;
			if ((idL == m_pickedID) || (idL == m_pickedID+1) || (idL+1 == m_pickedID)) 
			{
				pd[i].col1[0].x = (*m_cPID[idL]).x*wL + (*m_cPID[idL+1]).x*(1-wL);
				pd[i].col1[0].y = (*m_cPID[idL]).y*wL + (*m_cPID[idL+1]).y*(1-wL);
				pd[i].col1[0].z = (*m_cPID[idL]).z*wL + (*m_cPID[idL+1]).z*(1-wL);
				pd[i].col2[0].x = (*m_cPID2[idL]).x*wL + (*m_cPID2[idL+1]).x*(1-wL);
				pd[i].col2[0].y = (*m_cPID2[idL]).y*wL + (*m_cPID2[idL+1]).y*(1-wL);
				pd[i].col2[0].z = (*m_cPID2[idL]).z*wL + (*m_cPID2[idL+1]).z*(1-wL);
			}
			int idR = pd[i].IDw.z;
			float wR = pd[i].IDw.w;
			if ((idR == m_pickedID) || (idR == m_pickedID+1) || (idR+1 == m_pickedID)) 
			{
				pd[i].col1[1].x = (*m_cPID[idR]).x*wR + (*m_cPID[idR+1]).x*(1-wR);
				pd[i].col1[1].y = (*m_cPID[idR]).y*wR + (*m_cPID[idR+1]).y*(1-wR);
				pd[i].col1[1].z = (*m_cPID[idR]).z*wR + (*m_cPID[idR+1]).z*(1-wR);
				pd[i].col2[1].x = (*m_cPID2[idR]).x*wR + (*m_cPID2[idR+1]).x*(1-wR);
				pd[i].col2[1].y = (*m_cPID2[idR]).y*wR + (*m_cPID2[idR+1]).y*(1-wR);
				pd[i].col2[1].z = (*m_cPID2[idR]).z*wR + (*m_cPID2[idR+1]).z*(1-wR);
			}
		}
	}

	if (mode == BLUR_MODE)
	{
		float blurChange;
		float y = (mouse.y*m_sizeY - m_pickedMouse.y)*(m_sizeY/256.0f)/(m_sizeY);
		blurChange = -y; 
		float w = 1-m_pickedWBlur;
		*m_cPBID[m_pickedIDBlur] = max(0, m_pickedBlur1 + (1-w)*blurChange/(w*w+(1-w)*(1-w)));
		*m_cPBID[m_pickedIDBlur+1] = max(0, m_pickedBlur2 + w*blurChange/(w*w+(1-w)*(1-w)));
		for (int i=0; i<m_cSegNum*(int)(subSegNum)*10*2; i++)
		{
			int idLB = pd[i].IDwB.x;
			float wLB = pd[i].IDwB.y;
			if ((idLB == m_pickedIDBlur) || (idLB == m_pickedIDBlur+1) || (idLB+1 == m_pickedIDBlur)) 
			{
				pd[i].col1[0].w = (*m_cPBID[idLB])*wLB + (*m_cPBID[idLB+1])*(1-wLB);
				pd[i].col2[0].w = pd[i].col1[0].w;
			}
			int idRB = pd[i].IDwB.z;
			float wRB = pd[i].IDwB.w;
			if ((idRB == m_pickedIDBlur) || (idRB == m_pickedIDBlur+1) || (idRB+1 == m_pickedIDBlur)) 
			{
				pd[i].col1[1].w = (*m_cPBID[idRB])*wRB + (*m_cPBID[idRB+1])*(1-wRB);
				pd[i].col2[1].w = pd[i].col1[1].w;
			}
		}
	}
	if (mode == CURVE_MODE)
	{
		D3DXVECTOR2 changeVec;
		changeVec.x = 2.0*(mouse.x*m_sizeX - m_pickedMouse.x)/(m_sizeX*m_scale);
		changeVec.y = 2.0*(mouse.y*m_sizeY - m_pickedMouse.y)/(m_sizeY*m_scale);
		*m_cSplineID[m_pickedIDPos] = m_pickedPointPos+changeVec;
		for (int i=0; i<m_cSegNum*(int)(subSegNum)*10*2; i++)
		{
			int id = pd[i].IDwPos.x;
			float t = pd[i].IDwPos.y;
			if ((abs(id-m_pickedIDPos) <= 4)) 
			{
				pd[i].pos = (1.0f-t)*(1.0f-t)*(1.0f-t)*(*m_cSplineID[id+0]) + 3*t*(1.0f-t)*(1.0f-t)*(*m_cSplineID[id+1]) + 3*t*t*(1.0f-t)*(*m_cSplineID[id+2]) + t*t*t*(*m_cSplineID[id+3]);
				pd[i].pos.y = -pd[i].pos.y;
				if ((i>1) && ((i%2)==1) && (pd[i-2].nb.x < 9999.0f))
					pd[i-2].nb = pd[i].pos;
				else if ((i<m_cSegNum*(int)(subSegNum)*10*2-2) && ((i%2)==0) && (pd[i+2].nb.x < 9999.0f))
					pd[i+2].nb = pd[i].pos;
			}
		}
	}
	HRESULT hr;
	V( m_pMeshCurves->SetVertexData(0, pd) );
	V( m_pMeshCurves->CommitToDevice() );
}
