#pragma once

#define subSegNum 10.0  // each spline has 10 segments by definition, this number further subdivides each such segment
#define maxWNum 50	// maxWNum weight images, used to create the fitting

#define COLOR_MODE 1
#define LIGHTNESS_MODE 2
#define BLUR_MODE 3
#define CURVE_MODE 4

#include <vector>


struct SIMPLE_Vertex  // this is used for blur distance map drawing
{
    D3DXVECTOR2 pos; // Position
};

struct CONTROLPOINT_Vertex
{
	D3DXVECTOR2 pos;	// Position
	D3DXVECTOR3	col;	// the colors of this control point
};

struct VSO_Vertex
{
    D3DXVECTOR3 pos; // Position
	D3DXVECTOR2 tex; // Texturecoords
};


struct CURVE_Vertex
{
    D3DXVECTOR2 pos;	// Position
	D3DXVECTOR4 col1[2];// Color: left, right for the boundary
	D3DXVECTOR4 col2[2];// Color: left, right for the inner parts
	D3DXVECTOR2	nb;		// next vertex
	D3DXVECTOR4	IDw;	// ID and weight of the color interpolating vertices (first left then right)
	D3DXVECTOR4	IDwB;	// ID and weight of the blur interpolating vertices (first left then right)
	D3DXVECTOR2	IDwPos;	// ID and weight of the position interpolating vertices 
};

struct COLORPOINT
{
	D3DXVECTOR3 col;
	D3DXVECTOR3 col2;
	int			off;
	float		len;	// this is only needed for control point simplification
};

struct BLURRPOINT
{
	float		blurr;
	int			off;
};

struct CURVE
{
	D3DXVECTOR2 *p;
	int			pNum;
	COLORPOINT	*cl;
	int			clNum;
	COLORPOINT  *cr;
	int			crNum;
	BLURRPOINT  *bL;
	int			bLNum;
	BLURRPOINT  *bR;
	int			bRNum;
};



struct COLORPOINT_DENSE		// this is used for dense color point generation
{
	int			off;    // the offset for this control point along the curve
	D3DXVECTOR2	pos;	// position of that point in 2D
};




class VSObject
{
public:
	CComPtr<ID3DX10Mesh> m_pMeshDiff;
	CComPtr<ID3DX10Mesh> m_pMeshCurves;
	ID3DX10Mesh			*m_pMeshControlPoints;
	ID3DX10Mesh			*m_pDisplayMesh;

	ID3D10Texture2D *m_diffuseTexture[2];   // two textures used interleavedly for diffusion
	ID3D10Texture2D *m_distDirTexture;		// distance and direction texture 
	ID3D10Texture2D *m_resultTexture;		// result texture that combines all diffused data
	ID3D10Texture2D *m_pDepthStencil;       // used for z culling during curve rasterization 
	ID3D10Texture2D *m_otherTexture;		// texture that keeps the color on the _other_ side of a curve, used for anti aliasing
	ID3D10Texture2D *m_fitCPUTexture;



public:
	ID3D10ShaderResourceView *m_diffuseTextureRV[2];
	ID3D10RenderTargetView   *m_diffuseTextureTV[2];
	ID3D10ShaderResourceView *m_distDirTextureRV;
	ID3D10RenderTargetView   *m_distDirTextureTV;
	ID3D10ShaderResourceView *m_resultTextureRV;
	ID3D10RenderTargetView   *m_resultTextureTV;
	ID3D10DepthStencilView   *m_pDepthStencilView;
	ID3D10ShaderResourceView *m_otherTextureRV;
	ID3D10RenderTargetView   *m_otherTextureTV;
	ID3D10ShaderResourceView *m_paletteTextureRV;
	ID3D10ShaderResourceView *m_mousePointerTextureRV;

	ID3D10EffectTechnique *m_pDrawCurveDistancesTechnique;
	ID3D10EffectTechnique *m_pDrawCurveColorsTechnique;
	ID3D10EffectTechnique *m_pDiffuseDistancesTechnique;
	ID3D10EffectTechnique *m_pDistToYTechnique;
	ID3D10EffectTechnique *m_pBlurrDistTechnique;
	ID3D10EffectTechnique *m_pDiffuseColorsTechnique;
	ID3D10EffectTechnique *m_pAddToOutputTechnique;
	ID3D10EffectTechnique *m_pBlurTechnique;
	ID3D10EffectTechnique *m_pAntiAliasTechnique;
	ID3D10EffectTechnique *m_pDisplayImage;
	ID3D10EffectTechnique *m_pDrawCurvesOverlay;
	ID3D10EffectTechnique *m_pDisplayPalette;
	ID3D10EffectTechnique *m_pDisplayMouse;
	ID3D10EffectTechnique *m_pDrawControlPointTechnique;

	ID3D10EffectShaderResourceVariable* m_pInTex;
	ID3D10EffectShaderResourceVariable* m_pInTex2;
	ID3D10EffectShaderResourceVariable* m_pDistDirTex;
	ID3D10EffectShaderResourceVariable* m_pPaletteTex;
	ID3D10EffectShaderResourceVariable* m_pMousePointerTex;
	ID3D10EffectScalarVariable* m_pDiffX;
	ID3D10EffectScalarVariable* m_pDiffY;
	ID3D10EffectScalarVariable* g_pScale;
	ID3D10EffectVectorVariable* g_pPan;
	ID3D10EffectVectorVariable* g_pImageScale;  //defines for the blurr, how the scalar between the original image and the current image is
	ID3D10EffectScalarVariable* m_pPolySize;
	ID3D10EffectScalarVariable* m_pShowLayers;
	ID3D10EffectScalarVariable* m_pBlurScale;
	ID3D10EffectScalarVariable* m_pColLScale;
	ID3D10EffectScalarVariable* m_pColRScale;
	ID3D10EffectScalarVariable* m_pCmpColor;
	

	D3D10_VIEWPORT m_vp;

	CURVE *m_curve;
	CURVE_Vertex *pd;
	float m_polySize;
	int	m_cNum;
	int m_cPointNum;
	int m_cSegNum;
	int m_bPointNum;
	D3DXVECTOR3	**m_cPID,**m_cPID2; // reference to the color control points
	D3DXVECTOR2 **m_cSplineID;		// reference to the spline control points
	float		**m_cPBID;			// reference to the blur points
	CONTROLPOINT_Vertex	*m_controlPointData;

	int m_sizeX, m_sizeY;
	float m_fWidth;
	float m_fHeight;
	float m_scale;
	float m_blurScale;
	D3DXVECTOR2 m_pan;
	float m_1L,m_2L,m_1R,m_2R;

	int diffTex;
	int diffSteps;
	int diffSteps2;
	int diffSteps3;

	int			m_inputSizeX, m_inputSizeY;
	D3DXVECTOR3 *m_inputI;						// container for the input image info
	D3DXVECTOR2 *m_wImage[maxWNum];				// maxWNum weight images, used to create the fitting
	D3DXVECTOR2 *m_wImage2[maxWNum];				// maxWNum weight images, used to create the fitting
	float		*m_pPaletteImage;				// array that holds the color palette (must be 256x256)

	D3DXVECTOR2 m_pickedMouse;			// picked mouse position
	D3DXVECTOR2 m_pickedPalMouse;		// picked mouse position in palette
	D3DXVECTOR2 m_pickedPointPos;		// picked spline control point position
	D3DXVECTOR2 m_pickedPoint1;			// picked spline control point position
	D3DXVECTOR2 m_pickedPoint2;			// picked spline control point position
	D3DXVECTOR2 m_pickedPoint3;			// picked spline control point position
	D3DXVECTOR2 m_pickedPoint4;			// picked spline control point position
	D3DXVECTOR3 m_pickedPalCol;			// picked color in palette
	D3DXVECTOR3 m_pickedCol1L1;			// picked color 
	D3DXVECTOR3 m_pickedCol2L1;			// picked color 
	D3DXVECTOR3 m_pickedCol1L2;			// picked color 
	D3DXVECTOR3 m_pickedCol2L2;			// picked color 
	float		m_pickedBlurVal;		// picked blur value
	float		m_pickedBlur1;			// picked blur of control point 1
	float		m_pickedBlur2;			// picked blur of control point 2
	int			m_pickedID, m_pickedIDPos, m_pickedIDBlur;	// the picked control points
	float		m_pickedWeight, m_pickedWPos, m_pickedWBlur, m_pickedDepth;
	
	static D3D10_INPUT_ELEMENT_DESC InputElements[];
	static D3D10_INPUT_ELEMENT_DESC InputCurveElements[];
	static D3D10_INPUT_ELEMENT_DESC ScreenSpaceElements[];
	static D3D10_INPUT_ELEMENT_DESC ControlPointElementLayout[];
	static int InputElementCount;
	static int InputCurveElementCount;
	static int ScreenSpaceElementCount;
	static int InputControlPointElementCount;
	
	ID3D10Buffer *m_pVertexBuffer;
	ID3D10InputLayout* m_pVertexLayout;

	VSObject(ID3D10Device *pd3dDevice);
	~VSObject(void);

	BOOL CreateVertexBuffer(ID3D10Device* pd3dDevice);
	BOOL CreateIndexBuffer(ID3D10Device *pd3dDevice);
	int GetNearestSegmentID( float pX, float pY, int &id1, float &weight, int &idPos, float &wPos, int &idBlur, float &wBlur);
	void SetupTextures(ID3D10Device *pd3dDevice, ID3D10Effect* g_pEffect10, int sizeX, int sizeY);
	void LoadVectorFile(ID3D10Device *pd3dDevice, char *s );
	void ReadVectorFile( char *s );
	void VSObject::ComputeColorPointDistances();
	void ConstructCurves(ID3D10Device *pd3dDevice);
	void UpdateDisplayMesh(ID3D10Device *pd3dDevice);
	void RenderDiffusion(ID3D10Device *pd3dDevice, int vID, int L_1_2);   // L_1_2 means which Laplacian layer to render (0=both)
	void Render(ID3D10Device *pd3dDevice, bool showLayerWeight, bool showBlurMap, bool showBlurDist, bool showCurves, bool showControlPoints, int mode);

	void SaveGDCI(char *destColFile);

	// Palette functions
	void DisplayPalette(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse, int mode);
	void SetMousePicker(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse, int mode);
	void SetPickedColors(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse, int mode);
	void DisplayMouse(ID3D10Device *pd3dDevice, D3DXVECTOR2 mouse);

	void DumpScreenToFile(ID3D10Device *pd3dDevice, int fileNumber);
};
