//--------------------------------------------------------------------------------------
// File: VolSurfaces10.cpp
//
// This sample shows a simple example of the Microsoft Direct3D's High-Level 
// Shader Language (HLSL) using the Effect interface. 
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------
#define D3D_DEBUG_INFO

#include "Globals.h"
#include "DXUT.h"
#include "DXUTcamera.h"
#include "DXUTgui.h"
#include "DXUTsettingsDlg.h"
#include "SDKmisc.h"
#include "SDKMesh.h"
#include "VSObject.h"
#include "resource.h"
#include <string>

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
CDXUTDialogResourceManager g_DialogResourceManager; // manager for shared resources of dialogs
CModelViewerCamera      g_Camera;               // A model viewing camera
CDXUTDirectionWidget    g_LightControl;
CD3DSettingsDlg         g_D3DSettingsDlg;       // Device settings dialog
CDXUTDialog             g_HUD;                  // manages the 3D   
CDXUTDialog             g_SampleUI;             // dialog for sample specific controls
float					g_showMenue = true;
bool					g_mouseLButtonDown = false;
bool					m_showPoints = false;
char					s[255] = "Media\\frogGDCI_editedNew.xml";

bool					m_paletteModeR = false;
int						m_mode = COLOR_MODE;

// Direct3D9 resources
CDXUTTextHelper*        g_pTxtHelper = NULL;

// Direct3D10 resources
ID3D10Device*			g_device = NULL;
ID3DX10Font*            g_pFont10 = NULL;       
ID3DX10Sprite*          g_pSprite10 = NULL;
VSObject*				g_vsObj = NULL;
ID3D10RasterizerState*	g_pRasterState;
ID3D10Effect*           g_pEffect10 = NULL;


//--------------------------------------------------------------------------------------
// UI control IDs
//--------------------------------------------------------------------------------------
#define IDC_TOGGLEFULLSCREEN    1
#define IDC_TOGGLEREF           3
#define IDC_CHANGEDEVICE        4

#define IDC_STATIC_EDITING		20
#define IDC_MODE_COLOR          301
#define IDC_MODE_LIGHTNESS      302
#define IDC_MODE_BLUR           303
#define IDC_MODE_CURVE          304

#define IDC_SHOW_CURVES			12
#define IDC_SHOWPOINTS			122
#define IDC_SHOW				13
#define IDC_SHOW_WEIGHT			14
#define IDC_SHOW_OUTPUT			17
#define IDC_SHOW_L1				171
#define IDC_SHOW_L2				172

#define IDC_LOAD_DCI			200
#define IDC_SAVE_GDCI			405


//--------------------------------------------------------------------------------------
// Forward declarations 
//--------------------------------------------------------------------------------------
bool    CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext );
void    CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext );
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing, void* pUserContext );
void    CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext );
void    CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );
bool    CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext );
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext );
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext );
void    CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext );
void    CALLBACK OnD3D10DestroyDevice( void* pUserContext );
void    CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext );
void    InitApp();
void    RenderText();

//--------------------------------------------------------------------------------------
// Entry point to the program. Initializes everything and goes into a message processing 
// loop. Idle time is used to render the scene.
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // DXUT will create and use the best device (either D3D9 or D3D10) 
    // that is available on the system depending on which D3D callbacks are set below

    // Set DXUT callbacks
    DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackKeyboard( OnKeyboard );
    DXUTSetCallbackFrameMove( OnFrameMove );

    DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );

    InitApp();
    DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( L"Generalized Diffusion Curves" );
    DXUTCreateDevice( true, 800, 800 );
    DXUTMainLoop(); // Enter into the DXUT render loop

    return DXUTGetExitCode();
}


//--------------------------------------------------------------------------------------
// Initialize the app 
//--------------------------------------------------------------------------------------
void InitApp()
{
    // Initialize dialogs
    g_D3DSettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );
    g_SampleUI.Init( &g_DialogResourceManager );
	g_HUD.SetCallback( OnGUIEvent ); 
	g_SampleUI.SetCallback( OnGUIEvent ); 

	int iY = -250; 

	g_SampleUI.AddStatic( IDC_STATIC_EDITING, L"Editing mode", 35, iY += 48, 125, 22 );
	g_SampleUI.AddRadioButton( IDC_MODE_COLOR, 1, L"Colors (1)", 35, iY+=24, 125, 22, true, '1', false );
	g_SampleUI.AddRadioButton( IDC_MODE_LIGHTNESS, 1, L"Brightness (2)", 35, iY+=24, 125, 22, false, '2', false );
	g_SampleUI.AddRadioButton( IDC_MODE_BLUR, 1, L"Blur (3)", 35, iY+=24, 125, 22, false, '3', false );
	g_SampleUI.AddRadioButton( IDC_MODE_CURVE, 1, L"Geometry (4)", 35, iY+=24, 125, 22, false, '4', false );

	g_SampleUI.AddStatic( IDC_SHOW, L"Visualizations", 35, iY += 48, 125, 22 );
	g_SampleUI.AddRadioButton( IDC_SHOW_OUTPUT, 0, L"Output image", 35, iY += 32, 125, 22, true );
	g_SampleUI.AddCheckBox( IDC_SHOW_L1, L"L_1", 50, iY += 24, 50, 22, false );
	g_SampleUI.AddCheckBox( IDC_SHOW_L2, L"L_2", 100, iY += 0, 50, 22, false );
	g_SampleUI.AddRadioButton( IDC_SHOW_WEIGHT,  0, L"Weight W_l", 35, iY += 24, 125, 22, false );
	g_SampleUI.AddCheckBox( IDC_SHOW_CURVES, L"Curves", 35, iY += 24, 125, 22, false );
	g_SampleUI.AddCheckBox( IDC_SHOWPOINTS, L"Control points", 35, iY += 24, 125, 22, false, 'X', m_showPoints );

	g_SampleUI.AddButton(IDC_LOAD_DCI, L"Load curves", 35, iY+=36, 125, 22, false );
	g_SampleUI.AddButton(IDC_SAVE_GDCI, L"Save as GDCI", 35, iY+=24, 125, 22, false );
}


//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
	pDeviceSettings->d3d10.sd.SampleDesc.Count = 1;
	pDeviceSettings->d3d10.sd.SampleDesc.Quality = 0;

	if( DXUT_D3D9_DEVICE == pDeviceSettings->ver )
    {
        D3DCAPS9 Caps;
        IDirect3D9 *pD3D = DXUTGetD3D9Object();
        pD3D->GetDeviceCaps( pDeviceSettings->d3d9.AdapterOrdinal, pDeviceSettings->d3d9.DeviceType, &Caps );

        // If device doesn't support HW T&L or doesn't support 1.1 vertex shaders in HW 
        // then switch to SWVP.
        if( (Caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) == 0 ||
            Caps.VertexShaderVersion < D3DVS_VERSION(1,1) )
        {
            pDeviceSettings->d3d9.BehaviorFlags = D3DCREATE_SOFTWARE_VERTEXPROCESSING;
        }

        // Debugging vertex shaders requires either REF or software vertex processing 
        // and debugging pixel shaders requires REF.  
#ifdef DEBUG_VS
        if( pDeviceSettings->d3d9.DeviceType != D3DDEVTYPE_REF )
        {
            pDeviceSettings->d3d9.BehaviorFlags &= ~D3DCREATE_HARDWARE_VERTEXPROCESSING;
            pDeviceSettings->d3d9.BehaviorFlags &= ~D3DCREATE_PUREDEVICE;                            
            pDeviceSettings->BehaviorFlags |= D3DCREATE_SOFTWARE_VERTEXPROCESSING;
        }
#endif
#ifdef DEBUG_PS
        pDeviceSettings->d3d9.DeviceType = D3DDEVTYPE_REF;
#endif
    }
    else
    {
        // Uncomment this to get debug information from D3D10
        //pDeviceSettings->d3d10.CreateFlags |= D3D10_CREATE_DEVICE_DEBUG;
    }

    // For the first device created if its a REF device, optionally display a warning dialog box
    static bool s_bFirstTime = true;
    if( s_bFirstTime )
    {
        s_bFirstTime = false;
        if( (DXUT_D3D9_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d9.DeviceType == D3DDEVTYPE_REF) ||
            (DXUT_D3D10_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d10.DriverType == D3D10_DRIVER_TYPE_REFERENCE) )
            DXUTDisplaySwitchingToREFWarning( pDeviceSettings->ver );
    }

    return true;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
	g_Camera.FrameMove( fElapsedTime );
}


//--------------------------------------------------------------------------------------
// Render the help and statistics text
//--------------------------------------------------------------------------------------
void RenderText()
{
    g_pTxtHelper->Begin();
    g_pTxtHelper->SetInsertionPos( 2, 0 );
    g_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 0.0f, 0.0f, 1.0f ) );
    g_pTxtHelper->DrawTextLine( DXUTGetFrameStats( true ) );
//    g_pTxtHelper->DrawTextLine( DXUTGetDeviceStats() );    
	g_pTxtHelper->End();
}


//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing, void* pUserContext )
{
    // Pass messages to dialog resource manager calls so GUI state is updated correctly
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Pass messages to settings dialog if its active
    if( g_D3DSettingsDlg.IsActive() )
    {
        g_D3DSettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    // Give the dialogs a chance to handle the message first
    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;
    *pbNoFurtherProcessing = g_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    g_LightControl.HandleMessages( hWnd, uMsg, wParam, lParam );

    // Pass all remaining windows messages to camera so it can respond to user input
    g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam );

    return 0;
}


//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
	static int cVertex = 0;

    if( bKeyDown )
    {
       switch( nChar )
        {
            case 'M': g_showMenue = !g_showMenue; break;
		}
    }
}


//--------------------------------------------------------------------------------------
// Handles the GUI events
//--------------------------------------------------------------------------------------
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{   
    switch( nControlID )
    {
        case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen(); break;
        case IDC_TOGGLEREF:        DXUTToggleREF(); break;
        case IDC_CHANGEDEVICE:     g_D3DSettingsDlg.SetActive( !g_D3DSettingsDlg.IsActive() ); break;
 
		case IDC_SHOW_L1:			{
										if (g_SampleUI.GetCheckBox( IDC_SHOW_L1)->GetChecked())
											g_SampleUI.GetCheckBox( IDC_SHOW_L2)->SetChecked(false);
										break;
									}
		case IDC_SHOW_L2:			{
										if (g_SampleUI.GetCheckBox( IDC_SHOW_L2)->GetChecked())
											g_SampleUI.GetCheckBox( IDC_SHOW_L1)->SetChecked(false);
										break;
									}

		case IDC_SHOWPOINTS:		{
										m_showPoints = !m_showPoints;
										break;
									}

		// from here all items for curve coloring
		case IDC_MODE_COLOR:		{
										m_mode = COLOR_MODE; 
										break;
									}
		case IDC_MODE_LIGHTNESS:	{
										m_mode = LIGHTNESS_MODE; 
										break;
									}
		case IDC_MODE_BLUR:			{
										m_mode = BLUR_MODE; 
										break;
									}
		case IDC_MODE_CURVE:		{
										m_mode = CURVE_MODE; 
										break;
									}

		case IDC_LOAD_DCI:			{
										 // open a file
										WCHAR szRescue[300];
										for (int i=0; i<300; i++)
											szRescue[i] = s[i];
										OPENFILENAME ofnMesh;
										ZeroMemory(&ofnMesh, sizeof(ofnMesh));
										ofnMesh.lStructSize = sizeof ( ofnMesh );
										ofnMesh.hwndOwner = NULL;
										ofnMesh.lpstrFile = &szRescue[0];
										ofnMesh.lpstrFile[0] = '\0';
										ofnMesh.nMaxFile = sizeof(s);
										ofnMesh.lpstrFilter = L"All\0*.*\0";
										ofnMesh.nFilterIndex = 1;
										ofnMesh.lpstrFileTitle = NULL;
										ofnMesh.nMaxFileTitle = 0;
										ofnMesh.lpstrInitialDir = NULL;
										ofnMesh.Flags = OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
										GetOpenFileName( &ofnMesh );
										if(wcslen(ofnMesh.lpstrFile) == 0)
											break;
										for (int i=0; i<300; i++)
											s[i] = szRescue[i];
										g_vsObj->LoadVectorFile(g_device, &s[0] );
										break;
									}

		case IDC_SAVE_GDCI:		{
										// save as DCI file
										WCHAR szRescue[300];
										for (int i=0; i<300; i++)
											szRescue[i] = s[i];
										OPENFILENAME ofnMesh;
										ZeroMemory(&ofnMesh, sizeof(ofnMesh));
										ofnMesh.lStructSize = sizeof ( ofnMesh );
										ofnMesh.hwndOwner = NULL;
										ofnMesh.lpstrFile = &szRescue[0];
										ofnMesh.lpstrFile[0] = '\0';
										ofnMesh.nMaxFile = sizeof(s);
										ofnMesh.lpstrFilter = L"All\0*.*\0";
										ofnMesh.nFilterIndex = 1;
										ofnMesh.lpstrFileTitle = NULL;
										ofnMesh.nMaxFileTitle = 0;
										ofnMesh.lpstrInitialDir = NULL;
										ofnMesh.Flags = OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
										GetSaveFileName( &ofnMesh );
										if(wcslen(ofnMesh.lpstrFile) == 0)
											break;
										for (int i=0; i<300; i++)
											s[i] = szRescue[i];
										g_vsObj->SaveGDCI(&s[0]);
										break;
									}
	}
}


//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}



//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC *pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;

	g_device = pd3dDevice;

    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_D3DSettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, 
                                OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
                                L"Arial", &g_pFont10 ) );
    V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, &g_pSprite10 ) );
    g_pTxtHelper = new CDXUTTextHelper( NULL, NULL, g_pFont10, g_pSprite10, 15 );

    V_RETURN( CDXUTDirectionWidget::StaticOnD3D10CreateDevice( pd3dDevice ) );

    // Read the D3DX effect file
    WCHAR str[MAX_PATH];
	ID3D10Blob *pErrBlob = NULL;
    V_RETURN( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"Effect_Undistort.fx" ) );
    hr = D3DX10CreateEffectFromFile( str, NULL, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, pd3dDevice, NULL, NULL, &g_pEffect10, &pErrBlob, NULL );
	if(FAILED ( hr ))
	{
		std::string errStr((LPCSTR)pErrBlob->GetBufferPointer(), pErrBlob->GetBufferSize());
		WCHAR err[10240];
		MultiByteToWideChar(CP_ACP, 0, errStr.c_str(), (int)errStr.size(), err, errStr.size());
		MessageBox( NULL, (LPCWSTR)err, L"Error", MB_OK );
		return hr;
	}

	g_vsObj = new VSObject(g_device); 
	g_vsObj->LoadVectorFile(g_device, &s[0]);

    D3D10_RASTERIZER_DESC rasterizerState;
    rasterizerState.FillMode = D3D10_FILL_SOLID;
    rasterizerState.CullMode = D3D10_CULL_NONE;
    rasterizerState.FrontCounterClockwise = true;
    rasterizerState.DepthBias = false;
    rasterizerState.DepthBiasClamp = 0;
    rasterizerState.SlopeScaledDepthBias = 0;
    rasterizerState.DepthClipEnable = true;
    rasterizerState.ScissorEnable = false;
    rasterizerState.MultisampleEnable = false;
    rasterizerState.AntialiasedLineEnable = false;
    pd3dDevice->CreateRasterizerState( &rasterizerState, &g_pRasterState );

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;

    V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );
    V_RETURN( g_D3DSettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );

    // Setup the camera's projection parameters
    float fAspectRatio = 1.0;//pBackBufferSurfaceDesc->Width / (FLOAT)pBackBufferSurfaceDesc->Height;
    g_Camera.SetProjParams( D3DX_PI/4, fAspectRatio, 0.1f, 10.0f );
    g_Camera.SetWindow( pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height );
    g_Camera.SetButtonMasks( MOUSE_RIGHT_BUTTON, MOUSE_WHEEL, MOUSE_MIDDLE_BUTTON );

    g_HUD.SetLocation( pBackBufferSurfaceDesc->Width-170, 0 );
    g_HUD.SetSize( 170, 170 );
    g_SampleUI.SetLocation( pBackBufferSurfaceDesc->Width-170, pBackBufferSurfaceDesc->Height-300 );
    g_SampleUI.SetSize( 170, 300 );

	// resize the texture so that it fits to the current screen size
	UINT width = ( DXUTIsAppRenderingWithD3D9() ) ? DXUTGetD3D9BackBufferSurfaceDesc()->Width : DXUTGetDXGIBackBufferSurfaceDesc()->Width;
	UINT height = ( DXUTIsAppRenderingWithD3D9() ) ? DXUTGetD3D9BackBufferSurfaceDesc()->Height : DXUTGetDXGIBackBufferSurfaceDesc()->Height;
	g_vsObj->SetupTextures(pd3dDevice, g_pEffect10, width, height);

	return S_OK;
}


//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    HRESULT hr;
	
	// If the settings dialog is being shown, then render it instead of rendering the app's scene
	if( g_D3DSettingsDlg.IsActive() )
    {
        g_D3DSettingsDlg.OnRender( fElapsedTime );
        return;
    }

    if( g_Camera.m_nMouseWheelDelta && g_Camera.m_nZoomButtonMask == MOUSE_WHEEL )
	{
		g_vsObj->m_pan /= g_vsObj->m_scale;
		if (g_Camera.m_nMouseWheelDelta>0)
			g_vsObj->m_scale *= 1.2;
		else
			g_vsObj->m_scale /= 1.2;
		g_vsObj->m_pan *= g_vsObj->m_scale;
	    g_Camera.m_nMouseWheelDelta = 0;
		g_vsObj->m_scale = min(1000.0, max(0.01, g_vsObj->m_scale));
	}

	if ((!g_Camera.IsMouseRButtonDown()) && (g_mouseLButtonDown == true))
	{
		g_mouseLButtonDown = false;
	}
	else if (g_Camera.IsMouseRButtonDown())
		g_mouseLButtonDown = true;
	else
		g_mouseLButtonDown = false;

    if((g_Camera.IsBeingDragged()) && ((g_Camera.IsMouseLButtonDown()) ||(g_Camera.IsMouseMButtonDown())))
	{
		float xFac = 4.0f/g_vsObj->m_sizeX;
		float yFac = 4.0f/g_vsObj->m_sizeY;
		g_vsObj->m_pan += D3DXVECTOR2(xFac*g_Camera.m_vMouseDelta.x,-yFac*g_Camera.m_vMouseDelta.y);
		g_Camera.m_vMouseDelta.x = 0;
		g_Camera.m_vMouseDelta.y = 0;
	}

	g_vsObj->m_polySize = 2.3;
	int L_1_2 = 0;
	if (g_SampleUI.GetCheckBox(IDC_SHOW_L1)->GetChecked())
		L_1_2 = 1;
	if (g_SampleUI.GetCheckBox(IDC_SHOW_L2)->GetChecked())
		L_1_2 = 2;
	g_vsObj->RenderDiffusion(pd3dDevice, -1, L_1_2);
	g_vsObj->Render(pd3dDevice, g_SampleUI.GetRadioButton(IDC_SHOW_WEIGHT)->GetChecked(), false, false, g_SampleUI.GetCheckBox(IDC_SHOW_CURVES)->GetChecked(), m_showPoints, m_mode);


    if (g_Camera.IsMouseRButtonDown())
	{
		// Get the mouse coordinates in screen space
		POINT ptCursor;
		GetCursorPos( &ptCursor );
		ScreenToClient( DXUTGetHWND(), &ptCursor );
		float x = (float)(ptCursor.x)/(DXUTGetDXGIBackBufferSurfaceDesc()->Width);
		float y = (float)(ptCursor.y)/DXUTGetDXGIBackBufferSurfaceDesc()->Height;
		if (!m_paletteModeR) 
			g_vsObj->SetMousePicker(pd3dDevice, D3DXVECTOR2(x,y), m_mode);
		else 
			g_vsObj->SetPickedColors(pd3dDevice, D3DXVECTOR2(x,y), m_mode);
		m_paletteModeR = true;
	}
	else 
	{
		m_paletteModeR = false;
	}

	// draw the palette if in color picking mode
	if ((m_paletteModeR) && (m_mode != CURVE_MODE))
	{
		POINT ptCursor;
		GetCursorPos( &ptCursor );
		ScreenToClient( DXUTGetHWND(), &ptCursor );
		float x = (float)(ptCursor.x)/(DXUTGetDXGIBackBufferSurfaceDesc()->Width);
		float y = (float)(ptCursor.y)/DXUTGetDXGIBackBufferSurfaceDesc()->Height;
		g_vsObj->DisplayPalette(pd3dDevice, D3DXVECTOR2(x,y), m_mode);
	}

	if (g_showMenue)
	{
		DXUT_BeginPerfEvent( DXUT_PERFEVENTCOLOR, L"HUD / Stats" );
	    g_HUD.OnRender( fElapsedTime ); 
		g_SampleUI.OnRender( fElapsedTime );
 		RenderText();
		DXUT_EndPerfEvent();
	}
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10ReleasingSwapChain();
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10DestroyDevice();
    g_D3DSettingsDlg.OnD3D10DestroyDevice();
    CDXUTDirectionWidget::StaticOnD3D10DestroyDevice();
    DXUTGetGlobalResourceCache().OnDestroyDevice();
    SAFE_DELETE( g_pTxtHelper );
    SAFE_RELEASE( g_pFont10 );
    SAFE_RELEASE( g_pSprite10 );
    SAFE_RELEASE( g_pEffect10 );
	SAFE_DELETE( g_vsObj );
}



